@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -56,6 +56,7 @@ import org.eclipse.ui.console.IConsole;
 import org.eclipse.ui.console.IConsoleManager;
 import org.eclipse.ui.console.IOConsoleOutputStream;
 import org.eclipse.ui.console.MessageConsole;
+import org.osgi.framework.Version;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -92,10 +93,15 @@ public class ApplicationWorkbenchAdvisor extends WorkbenchAdvisor {
 			return false;
 		}
 
-		//long mx = -1;
+		Version curVers = Platform.getProduct().getDefiningBundle().getVersion();
+		Version iniVers = Version.parseVersion(System.getProperty("javie.ini.version"));
+
 		for (String arg : vmargs.split("\n|\r\n?")) {
 			if (arg.startsWith("-Xmx")) {
 				logger.info("-Xmx found in vmargs: " + arg);
+				if (iniVers.compareTo(new Version(0, 5, 1)) < 0) {
+					return updateJavieIni(0, curVers.toString(), iniVers.toString());
+				}
 				return false;
 			}
 		}
@@ -130,15 +136,27 @@ public class ApplicationWorkbenchAdvisor extends WorkbenchAdvisor {
 		} else if (physMemSize < 512*1000*1000) {
 			newMaxHeap = physMemSize*3/4 - 128*1000*1000;
 		} else {
-			newMaxHeap = Math.min(physMemSize/2, 1000*1000*1000);
+			newMaxHeap = Math.min(physMemSize/2, 512*1000*1000);
 		}
 
 		if (newMaxHeap <= curMaxHeap) {
 			return false;
 		}
 
-		String newXmx = "-Xmx" + newMaxHeap;
-		logger.info("Adding to vmargs: " + newXmx);
+		return updateJavieIni(newMaxHeap, curVers.toString(), iniVers.toString());
+	}
+
+	private boolean updateJavieIni(long newMaxHeap, String newIniVers, String oldIniVers) {
+		newIniVers = "-Djavie.ini.version=" + newIniVers;
+		logger.info("Adding to vmargs: " + newIniVers);
+
+		String newXmx;
+		if (newMaxHeap > 0) {
+			newXmx = "-Xmx" + newMaxHeap;
+			logger.info("Adding to vmargs: " + newXmx);
+		} else {
+			newXmx = null;
+		}
 
 		try {
 			File curIniFile = new File("Javie.ini");
@@ -151,22 +169,27 @@ public class ApplicationWorkbenchAdvisor extends WorkbenchAdvisor {
 				pr = new PrintWriter(newIniFile);
 				String line;
 				while ((line = br.readLine()) != null) {
-					pr.println(line);
-					if (line.trim().equals("-vmargs")) {
-						pr.println(newXmx);
-						added = true;
+					String trim = line.trim();
+					if (!trim.startsWith("-Xmx") && !trim.startsWith("-Djavie.ini.version=")) {
+						pr.println(line);
+						if (trim.equals("-vmargs")) {
+							pr.println(newIniVers);
+							if (newXmx != null) pr.println(newXmx);
+							added = true;
+						}
 					}
 				}
 				if (!added) {
 					pr.println("-vmargs");
-					pr.println(newXmx);
+					pr.println(newIniVers);
+					if (newXmx != null) pr.println(newXmx);
 				}
 			} finally {
 				if (pr != null) pr.close();
 				if (br != null) br.close();
 			}
-			if (curIniFile.renameTo(new File("Javie.ini.bak"))) {
-				logger.error("cannot rename Javie.ini to Javie.ini.bak");
+			if (curIniFile.renameTo(new File("Javie.ini."+oldIniVers))) {
+				logger.error("cannot rename Javie.ini to Javie.ini."+oldIniVers);
 			}
 			if (!newIniFile.renameTo(curIniFile)) {
 				logger.error("cannot rename Javie.ini.new to Javie.ini");
@@ -202,12 +225,20 @@ public class ApplicationWorkbenchAdvisor extends WorkbenchAdvisor {
 
 		final GLGlobal glGlobal = InjectorHolder.getInjector().getInstance(GLGlobal.class);
 		if (glGlobal.getInitError() == null) {
+			logger.info("\n\n***** OpenGL Report *****\n"
+					+ "Vendor: " + (glGlobal.isIntel() ? "Intel" :
+									glGlobal.isAmdAti() ? "AMD" :
+									glGlobal.isNvidia() ? "NVIDIA" : "Unknown") + "\n"
+					+ "Driver Version: " + glGlobal.getDriverVersion() + "\n\n"
+					+ glGlobal.getReport()
+					+ "***** End of OpenGL Report *****\n");
+
 			boolean intel = glGlobal.isIntel();
 
 			if (win32 || (macosx && intel)) {
 				// TODO これらの値を環境設定で変更できるようにする。
 				int poolSize = (win32 && intel) ? 8 : 20;
-				GLCapabilities canvasCap = glGlobal.getCanvasCapabilities();
+				GLCapabilities canvasCap = (GLCapabilities) glGlobal.getCanvasCapabilities().cloneMutable();
 				canvasCap.setDoubleBuffered(win32);
 
 				glGlobal.enterPoolMode(poolSize);
