@@ -24,21 +24,32 @@
 package ch.kuramo.javie.app.player;
 
 import java.awt.Color;
+import java.awt.Dimension;
 import java.awt.Frame;
 import java.awt.GridBagConstraints;
 import java.awt.GridBagLayout;
+import java.lang.reflect.InvocationTargetException;
 import java.util.LinkedList;
 import java.util.Queue;
+import java.util.concurrent.ConcurrentLinkedQueue;
 
+import javax.media.opengl.GL2;
+import javax.media.opengl.GLAutoDrawable;
+import javax.media.opengl.GLEventListener;
 import javax.media.opengl.awt.GLCanvas;
 
+import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.jface.action.ControlContribution;
+import org.eclipse.jface.dialogs.ProgressMonitorDialog;
+import org.eclipse.jface.operation.IRunnableWithProgress;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.awt.SWT_AWT;
 import org.eclipse.swt.custom.ScrolledComposite;
-import org.eclipse.swt.graphics.Point;
+import org.eclipse.swt.custom.StackLayout;
+import org.eclipse.swt.graphics.RGB;
 import org.eclipse.swt.widgets.Composite;
 import org.eclipse.swt.widgets.Control;
+import org.eclipse.swt.widgets.Display;
 
 import ch.kuramo.javie.app.InjectorHolder;
 import ch.kuramo.javie.core.JavieRuntimeException;
@@ -157,21 +168,75 @@ public class GLCanvasFactory {
 			throw new IllegalStateException("not in pool mode");
 		}
 
-		_holder = new Composite(parent, SWT.NULL) {
-			private final Point size = new Point(1, 1);
-			public Point computeSize(int wHint, int hHint, boolean changed) {
-				return size;
-			}
-		};
+		_holder = new Composite(parent, SWT.NULL);
+
+		final StackLayout layout = new StackLayout();
+		_holder.setLayout(layout);
+
+		final Display display = _holder.getDisplay();
+		final RGB bgRGB = display.getSystemColor(SWT.COLOR_DARK_GRAY).getRGB();
+
+		final Queue<Control> controls = new ConcurrentLinkedQueue<Control>();
 
 		for (int i = 0; i < _poolSize; ++i) {
-			GLCanvasRecord record = newRecord(parent);
+			final GLCanvasRecord record = newRecord(_holder);
+			record.glCanvas.setPreferredSize(new Dimension(1, 1));
+			record.glCanvas.addGLEventListener(new GLEventListener() {
+				public void init(GLAutoDrawable drawable) {}
+				public void dispose(GLAutoDrawable drawable) {}
+				public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {}
+				public void display(GLAutoDrawable drawable) {
+					GL2 gl = drawable.getGL().getGL2();
+					gl.glPushAttrib(GL2.GL_COLOR_BUFFER_BIT);
+					try {
+						gl.glClearColor(bgRGB.red/255f, bgRGB.green/255f, bgRGB.blue/255f, 1f);
+						gl.glClear(GL2.GL_COLOR_BUFFER_BIT);
+					} finally {
+						gl.glPopAttrib();
+					}
 
-			// display()を実行することでコンテキストの生成とリストの共有が行われる。
-			record.glCanvas.display();
+					record.glCanvas.removeGLEventListener(this);
 
-			releaseGLCanvas(record);
+					display.asyncExec(new Runnable() {
+						public void run() {
+							releaseGLCanvas(record);
+							layout.topControl = controls.poll();
+							_holder.layout();
+						}
+					});
+				}
+			});
+
+			controls.offer(record.scrolled);
 		}
+
+		display.asyncExec(new Runnable() {
+			public void run() {
+				layout.topControl = controls.poll();
+				_holder.layout();
+
+				try {
+					ProgressMonitorDialog dialog = new ProgressMonitorDialog(_holder.getShell());
+					dialog.run(true, false, new IRunnableWithProgress() {
+						public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
+							int prev = controls.size();
+							monitor.beginTask("GLCanvasを初期化中...", prev);
+							while (!controls.isEmpty()) {
+								int curr = controls.size();
+								monitor.worked(prev-curr);
+								prev = curr;
+								Thread.sleep(10);
+							}
+							monitor.done();
+						}
+					});
+				} catch (InvocationTargetException e) {
+					throw new JavieRuntimeException(e);
+				} catch (InterruptedException e) {
+					throw new JavieRuntimeException(e);
+				}
+			}
+		});
 	}
 
 	public ControlContribution createControlContribution() {
