@@ -134,7 +134,7 @@ public class LayerSlipOperation extends ProjectOperation {
 
 		for (Object[] o : layersAndStartTimes) {
 			Layer layer = (Layer) o[0];
-			Time newStartTime = ((Time) o[1]).add(deltaTime);
+			Time newStartTime = deltaTime.add((Time) o[1]);
 			Time oldStartTime = layer.getStartTime();
 			_entries.add(new Entry(layer.getId(), oldStartTime, newStartTime));
 
@@ -181,7 +181,7 @@ public class LayerSlipOperation extends ProjectOperation {
 
 			@SuppressWarnings("unchecked")
 			AnimatableValue<Object> tmp = (AnimatableValue<Object>) newAvalue;
-			tmp.putKeyframe(kf.time.add(deltaTime), kf.value, kf.interpolation);
+			tmp.putKeyframe(deltaTime.add(kf.time), kf.value, kf.interpolation);
 		}
 
 		for (int i = 0, n = newAvalues.length; i < n; ++i) {
@@ -268,7 +268,7 @@ public class LayerSlipOperation extends ProjectOperation {
 					restore(_newAvalues[i], avalue);
 				}
 				if (createData) {
-					data[i] = new Object[] { layer, effectIndex, property, kf.time.add(_deltaTime) };
+					data[i] = new Object[] { layer, effectIndex, property, _deltaTime.add(kf.time) };
 				}
 			}
 		}
@@ -285,7 +285,7 @@ public class LayerSlipOperation extends ProjectOperation {
 			dst.reset(src.getStaticValue());
 			dst.setExpression(src.getExpression());
 
-			for (Keyframe<V> kf : src.getKeyframes()) {
+			for (Keyframe<V> kf : src.getKeyframeMap().values()) {
 				dst.putKeyframe(kf);
 			}
 
