@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010 Yoshikazu Kuramochi
+ * Copyright (c) 2010,2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,31 +23,23 @@
 
 package ch.kuramo.javie.app.views.layercomp;
 
-import java.util.Collections;
-import java.util.List;
-
-import org.eclipse.jface.viewers.ArrayContentProvider;
-import org.eclipse.jface.viewers.CellEditor;
-import org.eclipse.jface.viewers.LabelProvider;
+import org.eclipse.jface.action.Action;
+import org.eclipse.jface.action.ActionContributionItem;
+import org.eclipse.jface.action.IAction;
 import org.eclipse.swt.SWT;
-import org.eclipse.swt.events.SelectionAdapter;
-import org.eclipse.swt.events.SelectionEvent;
 import org.eclipse.swt.graphics.GC;
-import org.eclipse.swt.widgets.Combo;
+import org.eclipse.swt.graphics.Rectangle;
+import org.eclipse.swt.widgets.Menu;
+import org.eclipse.swt.widgets.MenuItem;
 
-import ch.kuramo.javie.app.views.ComboBoxViewerCellEditor;
 import ch.kuramo.javie.app.views.LayerCompositionView;
 import ch.kuramo.javie.core.AnimatableLayerReference;
 import ch.kuramo.javie.core.ItemLayer;
 import ch.kuramo.javie.core.Layer;
 import ch.kuramo.javie.core.LayerComposition;
-import ch.kuramo.javie.core.Util;
 
 public class AnimatableLayerReferenceElementDelegate extends AnimatableValueElementDelegate<String> {
 
-	private ComboBoxViewerCellEditor editor;
-
-
 	public AnimatableLayerReferenceElementDelegate(
 			AnimatableValueElement element, String name, AnimatableLayerReference avalue) {
 
@@ -55,16 +47,7 @@ public class AnimatableLayerReferenceElementDelegate extends AnimatableValueElem
 	}
 
 	protected void drawValue(GC gc, int x, int y, int height, boolean focused) {
-		String value;
-		Layer layer = getReferencedLayer();
-		if (layer == null) {
-			value = "なし";
-		} else if (!(layer instanceof ItemLayer<?>) || isLayerNameMode()) {
-			value = layer.getName();
-		} else {
-			value = ((ItemLayer<?>) layer).getItem().getName();
-		}
-		drawValue(gc, x, y, height, focused, value);
+		drawValue(gc, x, y, height, focused, getName(getReferencedLayer()));
 	}
 
 	protected boolean canDragGestureEdit() {
@@ -75,45 +58,37 @@ public class AnimatableLayerReferenceElementDelegate extends AnimatableValueElem
 		throw new UnsupportedOperationException("drag gesture edit is not supported");
 	}
 
-	public boolean canEdit(int columnIndex) {
-		if (columnIndex == LayerCompositionView.VALUE_COL) {
-			return canInlineEdit;
-		}
-		return false;
-	}
+	protected void openInPlaceEditor() {
+		Rectangle r = valueArea.get(valueIndex);
+		Menu menu = element.createPopUpMenu(r.x-5, r.y+r.height+2);
 
-	public CellEditor getCellEditor(int columnIndex) {
-		if (columnIndex == LayerCompositionView.VALUE_COL) {
-			if (editor == null) {
-				editor = new ComboBoxViewerCellEditor(element.viewer.getTree(), SWT.READ_ONLY);
-				editor.setContenProvider(ArrayContentProvider.getInstance());
-				editor.setLabelProvider(new LayerLabelProvider());
-				editor.setActivationStyle(ComboBoxViewerCellEditor.DROP_DOWN_ON_PROGRAMMATIC_ACTIVATION);
+		final Layer currentLayer = getReferencedLayer();
 
-				((Combo) editor.getControl()).addSelectionListener(new SelectionAdapter() {
-					public void widgetSelected(SelectionEvent e) {
-						setCellEditorValue(LayerCompositionView.VALUE_COL, editor.getValue());
-					}
-				});
+		Action action = new Action(getName(null), IAction.AS_RADIO_BUTTON) {
+			public void run() {
+				if (currentLayer != null) {
+					modifyValue("");
+				}
 			}
-			editor.setInput(getLayers());
-			return editor;
-		}
-		return null;
-	}
+		};
+		action.setChecked(currentLayer == null);
+		new ActionContributionItem(action).fill(menu, -1);
 
-	public Object getCellEditorValue(int columnIndex) {
-		if (columnIndex == LayerCompositionView.VALUE_COL) {
-			Layer layer = getReferencedLayer();
-			return (layer != null) ? layer : NO_LAYER;
-		}
-		return null;
-	}
+		new MenuItem(menu, SWT.SEPARATOR);
 
-	public void setCellEditorValue(int columnIndex, Object value) {
-		if (columnIndex == LayerCompositionView.VALUE_COL) {
-			modifyValue((value != NO_LAYER) ? ((Layer) value).getId() : "");
+		for (final Layer layer : getLayerComposition().getLayers()) {
+			action = new Action(getName(layer), IAction.AS_RADIO_BUTTON) {
+				public void run() {
+					if (currentLayer != layer) {
+						modifyValue(layer.getId());
+					}
+				}
+			};
+			action.setChecked(currentLayer == layer);
+			new ActionContributionItem(action).fill(menu, 2);
 		}
+
+		menu.setVisible(true);
 	}
 
 	private LayerComposition getLayerComposition() {
@@ -128,30 +103,13 @@ public class AnimatableLayerReferenceElementDelegate extends AnimatableValueElem
 		return Boolean.TRUE.equals(element.viewer.getData(LayerCompositionView.LAYER_NAME_MODE));
 	}
 
-	private Object[] getLayers() {
-		List<Object> layers = Util.newList();
-
-		for (Layer l :  getLayerComposition().getLayers()) {
-			layers.add(l);
-		}
-
-		layers.add(NO_LAYER);
-		Collections.reverse(layers);
-
-		return layers.toArray();
-	}
-
-	private static final Object NO_LAYER = new Object();
-
-	private class LayerLabelProvider extends LabelProvider {
-		public String getText(Object element) {
-			if (element == NO_LAYER) {
-				return "なし";
-			} else if (!(element instanceof ItemLayer<?>) || isLayerNameMode()) {
-				return ((Layer) element).getName();
-			} else {
-				return ((ItemLayer<?>) element).getItem().getName();
-			}
+	private String getName(Layer layer) {
+		if (layer == null) {
+			return "なし";
+		} else if (!(layer instanceof ItemLayer<?>) || isLayerNameMode()) {
+			return layer.getName();
+		} else {
+			return ((ItemLayer<?>) layer).getItem().getName();
 		}
 	}
 
