@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -30,18 +30,23 @@ import java.util.Map;
 import java.util.SortedMap;
 
 import org.eclipse.jface.viewers.CellEditor;
-import org.eclipse.jface.viewers.TextCellEditor;
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.events.DisposeEvent;
+import org.eclipse.swt.events.DisposeListener;
 import org.eclipse.swt.events.MouseEvent;
 import org.eclipse.swt.graphics.Cursor;
+import org.eclipse.swt.graphics.Font;
 import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.graphics.Point;
 import org.eclipse.swt.graphics.Rectangle;
+import org.eclipse.swt.layout.FillLayout;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Display;
 import org.eclipse.swt.widgets.Event;
 import org.eclipse.swt.widgets.Listener;
+import org.eclipse.swt.widgets.Shell;
+import org.eclipse.swt.widgets.Text;
 import org.eclipse.swt.widgets.TreeItem;
 
 import ch.kuramo.javie.api.Time;
@@ -51,6 +56,7 @@ import ch.kuramo.javie.app.project.ProjectManager;
 import ch.kuramo.javie.app.project.ProjectOperation;
 import ch.kuramo.javie.app.project.RemoveKeyframesOperation;
 import ch.kuramo.javie.app.views.LayerCompositionView;
+import ch.kuramo.javie.app.widgets.InPlaceEditorShell;
 import ch.kuramo.javie.core.AnimatableValue;
 import ch.kuramo.javie.core.ArithmeticalAnimatableValue;
 import ch.kuramo.javie.core.JavieRuntimeException;
@@ -89,7 +95,7 @@ public abstract class AnimatableValueElementDelegate<V> {
 
 	protected DragGestureEditor dragGestureEditor;
 
-	protected boolean canInlineEdit;
+	protected boolean canInPlaceEdit;
 
 	protected Element[] children;
 
@@ -157,13 +163,15 @@ public abstract class AnimatableValueElementDelegate<V> {
 
 				valueArea.clear();
 
-				GC gc = event.gc;
-				int x = event.x + 5;
-				int y = event.y;
-				int height = ((TreeItem) event.item).getBounds(event.index).height;
-				boolean focused = ((event.detail & SWT.SELECTED) != 0 && ((Control) event.widget).isFocusControl());
-
-				drawValue(gc, x, y, height, focused);
+				if (value != null) {
+					GC gc = event.gc;
+					int x = event.x + 5;
+					int y = event.y;
+					int height = ((TreeItem) event.item).getBounds(event.index).height;
+					boolean focused = ((event.detail & SWT.SELECTED) != 0 && ((Control) event.widget).isFocusControl());
+	
+					drawValue(gc, x, y, height, focused);
+				}
 				break;
 
 			case LayerCompositionView.SHOWHIDE_COL:
@@ -207,7 +215,7 @@ public abstract class AnimatableValueElementDelegate<V> {
 						if (r.contains(event.x, event.y)) {
 							if (!canDragGestureEdit()) {
 								valueIndex = i;
-								openInlineEditor();
+								openInPlaceEditor();
 							} else if (dragGestureEditor == null) {
 								originalValue = value;
 								valueIndex = i;
@@ -265,12 +273,12 @@ public abstract class AnimatableValueElementDelegate<V> {
 		return (avalue.getExpression() != null);
 	}
 
-	protected void openInlineEditor() {
+	protected void openInPlaceEditor() {
 		element.viewer.getTree().setCursor(null);
 
-		canInlineEdit = true;
+		canInPlaceEdit = true;
 		element.viewer.editElement(element, LayerCompositionView.VALUE_COL);
-		canInlineEdit = false;
+		canInPlaceEdit = false;
 	}
 
 	protected void modifyValue(V newValue) {
@@ -509,7 +517,7 @@ public abstract class AnimatableValueElementDelegate<V> {
 							display.removeFilter(SWT.Deactivate, this);
 
 							if (e.type == SWT.MouseUp && !control.isDisposed() && !dragDetected) {
-								openInlineEditor();
+								openInPlaceEditor();
 							}
 							break;
 					}
@@ -526,7 +534,9 @@ public abstract class AnimatableValueElementDelegate<V> {
 
 	public class ExpressionElement extends Element {
 
-		private static final String EDITOR_KEY = "ExpressionElement.TextCellEditor";
+		private Rectangle exprArea;
+
+		private Point editorSize;
 
 
 		public ExpressionElement() {
@@ -539,63 +549,84 @@ public abstract class AnimatableValueElementDelegate<V> {
 					return "エクスプレッション：" + name;
 
 				case LayerCompositionView.TIMELINE_COL:
-					return avalue.getExpression();
+					return avalue.getExpression().replaceAll("\\r?\\n|\\r", " ");
 
 				default:
-					return null;
+					return super.getColumnText(columnIndex);
 			}
 		}
 
+		public void paintColumn(Event event) {
+			if (event.index == LayerCompositionView.TIMELINE_COL) {
+				exprArea = ((TreeItem) event.item).getBounds(LayerCompositionView.TIMELINE_COL);
+			}
+			super.paintColumn(event);
+		}
+
 		public void mouseDown(MouseEvent event, int columnIndex) {
 			super.mouseDown(event, columnIndex);
 
-			// WIN32では選択中の行でしかエディタが起動しないので、プログラム上からエディタを起動する。
-			// (おそらくFULL_SELECTIONにすれば選択中でなくても(行のどこでもクリックすれば行が選択されるので、結果的に)エディタは起動するはずだが)
-
-			if (WIN32 && columnIndex == LayerCompositionView.TIMELINE_COL && event.button == 1) {
-				viewer.editElement(this, LayerCompositionView.TIMELINE_COL);
+			if (columnIndex == LayerCompositionView.TIMELINE_COL && event.button == 1) {
+				openInPlaceEditor();
 			}
 		}
 
-		public boolean canEdit(int columnIndex) {
-			if (columnIndex == LayerCompositionView.TIMELINE_COL) {
-				return true;
-			}
-			return false;
-		}
-
-		public CellEditor getCellEditor(int columnIndex) {
-			if (columnIndex == LayerCompositionView.TIMELINE_COL) {
-				TextCellEditor editor = (TextCellEditor) viewer.getData(EDITOR_KEY);
-				if (editor == null) {
-					editor = new TextCellEditor(viewer.getTree(), SWT.MULTI | SWT.V_SCROLL | SWT.BORDER) {
-						public LayoutData getLayoutData() {
-							LayoutData data = super.getLayoutData();
-							data.minimumHeight = 80;
-							data.verticalAlignment = SWT.TOP;
-							return data;
+		public void openInPlaceEditor() {
+			if (exprArea == null) {
+				viewer.getTree().getDisplay().asyncExec(new Runnable() {
+					public void run() {
+						if (exprArea != null) {
+							openInPlaceEditor();
 						}
-					};
-					Control control = editor.getControl();
-					control.setBackground(control.getDisplay().getSystemColor(SWT.COLOR_WHITE));
-					viewer.setData(EDITOR_KEY, editor);
+					}
+				});
+				return;
+			}
+
+			if (editorSize == null) {
+				editorSize = new Point(exprArea.width, 200);
+			}
+
+			final Shell shell = InPlaceEditorShell.create(
+					viewer.getTree(), exprArea,
+					editorSize.x, editorSize.y, true);
+
+			shell.setMinimumSize(100, 100);
+
+			FillLayout fillLayout = new FillLayout();
+			fillLayout.marginWidth = 5;
+			fillLayout.marginHeight = 5;
+			shell.setLayout(fillLayout);
+
+			final Text text = new Text(shell, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
+			text.setTabs(4);
+
+			final Font font;
+			if (WIN32) {
+				font = new Font(text.getDisplay(), "MS Gothic", 10, SWT.NORMAL);
+				text.setFont(font);
+			} else if (COCOA) {
+				font = new Font(text.getDisplay(), "Monaco", 11, SWT.NORMAL);
+				text.setFont(font);
+			} else {
+				font = null;
+			}
+
+			text.setText(avalue.getExpression());
+			text.selectAll();
+
+			shell.addDisposeListener(new DisposeListener() {
+				public void widgetDisposed(DisposeEvent e) {
+					modifyExpression(text.getText());
+					editorSize = shell.getSize();
+					if (font != null) {
+						text.setFont(null);
+						font.dispose();
+					}
 				}
-				return editor;
-			}
-			return null;
-		}
+			});
 
-		public Object getCellEditorValue(int columnIndex) {
-			if (columnIndex == LayerCompositionView.TIMELINE_COL) {
-				return avalue.getExpression();
-			}
-			return null;
-		}
-
-		public void setCellEditorValue(int columnIndex, Object value) {
-			if (columnIndex == LayerCompositionView.TIMELINE_COL) {
-				modifyExpression((String) value);
-			}
+			shell.open();
 		}
 
 	}
@@ -715,7 +746,7 @@ public abstract class AnimatableValueElementDelegate<V> {
 				if (element instanceof LayerAnimatableValueElement) {
 					LayerAnimatableValueElement avalueElem = (LayerAnimatableValueElement) element;
 					String id = avalueElem.layer.getId();
-					String prop = avalueElem.property;
+					String prop = avalueElem.getProperty();
 
 					for (Keyframe<V> kf : keyframes) {
 						data.add(new Object[] { id, -1, prop, kf });
