@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -42,14 +42,14 @@ import ch.kuramo.javie.app.views.LayerCompositionView;
 import ch.kuramo.javie.core.AnimatableVec2d;
 import ch.kuramo.javie.core.Layer;
 
-public class LayerAudioLevelElement extends LayerAnimatableValueElement {
+public class LayerAudioLevelElement extends LayerAnimatableValueElement.ForSimpleProperty {
 
 	public LayerAudioLevelElement(Element parent, Layer layer) {
 		super(parent, layer, "audioLevels", "オーディオレベル", "db");
 	}
 
 	protected AnimatableValueElementDelegate<?> createDelegate() {
-		AnimatableVec2d avalue = PropertyUtil.getProperty(layer, property);
+		AnimatableVec2d avalue = PropertyUtil.getProperty(layer, getProperty());
 		return new LayerAudioLevelElementDelegate(this, name, avalue, unit);
 	}
 
