@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,24 +23,27 @@
 
 package ch.kuramo.javie.app.views.layercomp;
 
-import java.util.Arrays;
-import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 
-import org.eclipse.jface.viewers.ArrayContentProvider;
+import org.eclipse.jface.action.Action;
+import org.eclipse.jface.action.ActionContributionItem;
+import org.eclipse.jface.action.IAction;
 import org.eclipse.jface.viewers.CellEditor;
-import org.eclipse.jface.viewers.LabelProvider;
 import org.eclipse.jface.viewers.TextCellEditor;
 import org.eclipse.jface.viewers.TreeViewer;
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.events.MouseEvent;
-import org.eclipse.swt.events.SelectionAdapter;
-import org.eclipse.swt.events.SelectionEvent;
+import org.eclipse.swt.graphics.Cursor;
+import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Image;
-import org.eclipse.swt.widgets.Combo;
+import org.eclipse.swt.graphics.Point;
+import org.eclipse.swt.graphics.Rectangle;
 import org.eclipse.swt.widgets.Control;
 import org.eclipse.swt.widgets.Event;
+import org.eclipse.swt.widgets.Menu;
+import org.eclipse.swt.widgets.MenuItem;
+import org.eclipse.swt.widgets.TreeItem;
 import org.eclipse.ui.ISharedImages;
 import org.eclipse.ui.PlatformUI;
 
@@ -52,7 +55,6 @@ import ch.kuramo.javie.app.project.ModifyLayerParentOperation;
 import ch.kuramo.javie.app.project.ModifyLayerTrackMatteOperation;
 import ch.kuramo.javie.app.project.ProjectManager;
 import ch.kuramo.javie.app.project.RenameLayerOperation;
-import ch.kuramo.javie.app.views.ComboBoxViewerCellEditor;
 import ch.kuramo.javie.app.views.LayerCompositionView;
 import ch.kuramo.javie.app.widgets.WSWin32;
 import ch.kuramo.javie.core.CameraLayer;
@@ -73,16 +75,11 @@ import ch.kuramo.javie.core.Util;
 
 public class LayerElement extends Element {
 
+	private static final int[] UNDERLINE_DASH = new int[] { 2, 2 };
+
 	private static final boolean COCOA = SWT.getPlatform().equals("cocoa");
 
-
-	private static final BlendMode[] layerBlendModes;
-
-	static{
-		List<BlendMode> list = Util.newList(Arrays.asList(BlendMode.values()));
-		list.remove(BlendMode.NONE);
-		layerBlendModes = list.toArray(new BlendMode[list.size()]);
-	}
+	private static final boolean WIN32 = SWT.getPlatform().equals("win32");
 
 
 	public final Layer layer;
@@ -99,11 +96,11 @@ public class LayerElement extends Element {
 
 	private TextCellEditor nameEditor;
 
-	private ComboBoxViewerCellEditor modeEditor;
+	private Rectangle modeArea;
 
-	private ComboBoxViewerCellEditor trackMatteEditor;
+	private Rectangle trackMatteArea;
 
-	private ComboBoxViewerCellEditor parentEditor;
+	private Rectangle parentArea;
 
 
 	public LayerElement(TreeViewer viewer, Layer layer) {
@@ -221,31 +218,6 @@ public class LayerElement extends Element {
 				return name;
 			}
 
-			case LayerCompositionView.MODE_COL:
-				return (layer instanceof MediaLayer)
-						? (COCOA ? "  " : "") + EnumLabels.getLabel(((MediaLayer) layer).getBlendMode()) : null;
-
-			case LayerCompositionView.TRACKMATTE_COL:
-				if (layer instanceof MediaLayer && getTrackMatteLayer() != null) {
-					TrackMatte trackMatte = ((MediaLayer) layer).getTrackMatte();
-					String label = EnumLabels.getLabel(trackMatte);
-					return (COCOA ? "  " : "") + (trackMatte != TrackMatte.NONE ? label : " - ");
-				}
-				return null;
-
-			case LayerCompositionView.PARENT_COL: {
-				String parentName;
-				Layer parent = composition.getParentLayer(layer);
-				if (parent == null) {
-					parentName = " - ";
-				} else if (!(parent instanceof ItemLayer<?>) || isLayerNameMode()) {
-					parentName = parent.getName();
-				} else {
-					parentName = ((ItemLayer<?>) parent).getItem().getName();
-				}
-				return (COCOA ? "  " : "") + parentName;
-			}
-
 			default:
 				return null;
 		}
@@ -294,7 +266,7 @@ public class LayerElement extends Element {
 			children.add(getChildElement(TextElement.class));
 		}
 
-		if (layer instanceof EffectableLayer && !((EffectableLayer) layer).getEffects().isEmpty()) {
+		if (layer instanceof EffectableLayer /*&& !((EffectableLayer) layer).getEffects().isEmpty()*/) {
 			children.add(getChildElement(EffectsElement.class));
 		}
 
@@ -347,6 +319,35 @@ public class LayerElement extends Element {
 				switchesColumnSwitches.paint(event);
 				break;
 
+			case LayerCompositionView.MODE_COL:
+				if (layer instanceof MediaLayer) {
+					modeArea = drawString(event,
+							EnumLabels.getLabel(((MediaLayer) layer).getBlendMode()));
+				}
+				break;
+
+			case LayerCompositionView.TRACKMATTE_COL:
+				if (layer instanceof MediaLayer && getTrackMatteLayer() != null) {
+					TrackMatte trackMatte = ((MediaLayer) layer).getTrackMatte();
+					trackMatteArea = drawString(event,
+							trackMatte != TrackMatte.NONE ? EnumLabels.getLabel(trackMatte) : "      ");
+				} else {
+					trackMatteArea = null;
+				}
+				break;
+
+			case LayerCompositionView.PARENT_COL: {
+				String parentName;
+				Layer parent = composition.getParentLayer(layer);
+				if (parent == null) {
+					parentName = "      ";
+				} else {
+					parentName = getName(parent);
+				}
+				parentArea = drawString(event, parentName);
+				break;
+			}
+
 			case LayerCompositionView.TIMELINE_COL:
 				TimelineManager tm = (TimelineManager) viewer.getData(LayerCompositionView.TIMELINE_MANAGER);
 				tm.drawLayer(event, layer);
@@ -358,17 +359,67 @@ public class LayerElement extends Element {
 		}
 	}
 
+	private Rectangle drawString(Event event, String str) {
+		GC gc = event.gc;
+		int x = event.x + 5;
+		int y = event.y;
+		int height = ((TreeItem) event.item).getBounds(event.index).height;
+		boolean focused = ((event.detail & SWT.SELECTED) != 0 && ((Control) event.widget).isFocusControl());
+
+		gc.setForeground(gc.getDevice().getSystemColor(
+				focused ? SWT.COLOR_WHITE : SWT.COLOR_DARK_BLUE));
+
+		Point extent = gc.textExtent(str, SWT.DRAW_TRANSPARENT);
+
+		y += (height - extent.y) / 2;
+		gc.drawString(str, x, y, true);
+
+		Rectangle area = new Rectangle(x, y, extent.x, extent.y);
+
+		y += extent.y;
+		gc.setLineDash(UNDERLINE_DASH);
+		if (COCOA) {
+			gc.drawLine(x, y - 1, x + extent.x, y - 1);
+		} else if (WIN32) {
+			gc.drawLine(x, y, x + extent.x - 2, y);
+		} else {
+			gc.drawLine(x, y, x + extent.x, y);
+		}
+
+		return area;
+	}
+
 	public void updateCursor(MouseEvent event, int columnIndex) {
+		Rectangle area;
+
 		switch (columnIndex) {
 			case LayerCompositionView.TIMELINE_COL:
 				TimelineManager tm = (TimelineManager) viewer.getData(LayerCompositionView.TIMELINE_MANAGER);
 				tm.updateCursor(event, layer);
-				break;
+				return;
 
 			default:
 				super.updateCursor(event, columnIndex);
+				return;
+
+			case LayerCompositionView.MODE_COL:
+				area = modeArea;
+				break;
+
+			case LayerCompositionView.TRACKMATTE_COL:
+				area = trackMatteArea;
+				break;
+
+			case LayerCompositionView.PARENT_COL:
+				area = parentArea;
 				break;
 		}
+
+		Cursor cursor = null;
+		if (area != null && area.contains(event.x, event.y)) {
+			cursor = event.display.getSystemCursor(SWT.CURSOR_HAND);
+		}
+		viewer.getTree().setCursor(cursor);
 	}
 
 	public void mouseDown(MouseEvent event, int columnIndex) {
@@ -381,18 +432,23 @@ public class LayerElement extends Element {
 				switchesColumnSwitches.mouseDown(event);
 				break;
 
-			case LayerCompositionView.TRACKMATTE_COL:
-				if (!(layer instanceof MediaLayer) || getTrackMatteLayer() == null) break;
-				// fall through
 			case LayerCompositionView.MODE_COL:
-				if (!(layer instanceof MediaLayer)) break;
-				// fall through
+				if (/*layer instanceof MediaLayer &&*/
+						modeArea != null && modeArea.contains(event.x, event.y)) {
+					showModeMenu();
+				}
+				break;
+
+			case LayerCompositionView.TRACKMATTE_COL:
+				if (/*layer instanceof MediaLayer && getTrackMatteLayer() != null &&*/
+						trackMatteArea != null && trackMatteArea.contains(event.x, event.y)) {
+					showTrackMatteMenu();
+				}
+				break;
+
 			case LayerCompositionView.PARENT_COL:
-				// WIN32では選択中の行でしかエディタが起動しないので、プログラム上からエディタを起動する。
-				// (おそらくFULL_SELECTIONにすれば選択中でなくても(行のどこでもクリックすれば行が選択されるので、結果的に)エディタは起動するはずだが)
-				// COCOAではプログラム上からエディタを起動しないと挙動がおかしくなる。
-				if (event.button == 1) {
-					viewer.editElement(this, columnIndex);
+				if (parentArea != null && parentArea.contains(event.x, event.y)) {
+					showParentMenu();
 				}
 				break;
 
@@ -407,20 +463,117 @@ public class LayerElement extends Element {
 		}
 	}
 
+	private Menu createPopUpMenu(Rectangle area) {
+		return createPopUpMenu(area.x-5, area.y+area.height+2);
+	}
+
+	private void showModeMenu() {
+		Menu menu = createPopUpMenu(modeArea);
+
+		final BlendMode currentMode = ((MediaLayer) layer).getBlendMode();
+
+		for (final BlendMode mode : BlendMode.values()) {
+			if (mode == BlendMode.NONE) continue;
+			if (EnumLabels.insertSeparator(mode)) {
+				new MenuItem(menu, SWT.SEPARATOR);
+			}
+			Action action = new Action(EnumLabels.getLabel(mode), IAction.AS_RADIO_BUTTON) {
+				public void run() {
+					if (currentMode != mode) {
+						ProjectManager pm = (ProjectManager) viewer.getData(LayerCompositionView.PROJECT_MANAGER);
+						pm.postOperation(new ModifyLayerBlendModeOperation(pm, (MediaLayer) layer, mode));
+					}
+				}
+			};
+			action.setChecked(currentMode == mode);
+			new ActionContributionItem(action).fill(menu, -1);
+		}
+
+		menu.setVisible(true);
+	}
+
+	private void showTrackMatteMenu() {
+		Menu menu = createPopUpMenu(trackMatteArea);
+
+		final TrackMatte current = ((MediaLayer) layer).getTrackMatte();
+
+		for (final TrackMatte tm : TrackMatte.values()) {
+			if (EnumLabels.insertSeparator(tm)) {
+				new MenuItem(menu, SWT.SEPARATOR);
+			}
+
+			String label;
+			if (tm == TrackMatte.NONE) {
+				label = EnumLabels.getLabel(tm);
+			} else {
+				label = String.format("%s - %s", EnumLabels.getLabel(tm), getName(getTrackMatteLayer()));
+			}
+
+			Action action = new Action(label, IAction.AS_RADIO_BUTTON) {
+				public void run() {
+					if (current != tm) {
+						ProjectManager pm = (ProjectManager) viewer.getData(LayerCompositionView.PROJECT_MANAGER);
+						pm.postOperation(new ModifyLayerTrackMatteOperation(pm, (MediaLayer) layer, tm));
+					}
+				}
+			};
+			action.setChecked(current == tm);
+			new ActionContributionItem(action).fill(menu, -1);
+		}
+
+		menu.setVisible(true);
+	}
+
+	private void showParentMenu() {
+		Menu menu = createPopUpMenu(parentArea);
+
+		final Layer currentParent = composition.getParentLayer(layer);
+
+		Action action = new Action(getName(null), IAction.AS_RADIO_BUTTON) {
+			public void run() {
+				if (currentParent != null) {
+					ProjectManager pm = (ProjectManager) viewer.getData(LayerCompositionView.PROJECT_MANAGER);
+					pm.postOperation(new ModifyLayerParentOperation(pm, layer, null));
+				}
+			}
+		};
+		action.setChecked(currentParent == null);
+		new ActionContributionItem(action).fill(menu, -1);
+
+		new MenuItem(menu, SWT.SEPARATOR);
+
+		for (final Layer otherLayer : composition.getLayers()) {
+			action = new Action(getName(otherLayer), IAction.AS_RADIO_BUTTON) {
+				public void run() {
+					if (currentParent != otherLayer) {
+						ProjectManager pm = (ProjectManager) viewer.getData(LayerCompositionView.PROJECT_MANAGER);
+						pm.postOperation(new ModifyLayerParentOperation(pm, layer, otherLayer));
+					}
+				}
+			};
+			action.setEnabled(otherLayer != layer);
+			action.setChecked(currentParent == otherLayer);
+			new ActionContributionItem(action).fill(menu, 2);
+		}
+
+		menu.setVisible(true);
+	}
+
+	private String getName(Layer layer) {
+		if (layer == null) {
+			return "なし";
+		} else if (!(layer instanceof ItemLayer<?>) || isLayerNameMode()) {
+			return layer.getName();
+		} else {
+			return ((ItemLayer<?>) layer).getItem().getName();
+		}
+	}
+
 	public boolean canEdit(int columnIndex) {
 		switch (columnIndex) {
 			case LayerCompositionView.NAME_COL:
 				return Boolean.TRUE.equals(viewer.getData(LayerCompositionView.EDIT_ELEMENT_NAME));
 
-			case LayerCompositionView.MODE_COL:
-				return (layer instanceof MediaLayer);
-
-			case LayerCompositionView.TRACKMATTE_COL:
-				return (layer instanceof MediaLayer && getTrackMatteLayer() != null);
-
-			case LayerCompositionView.PARENT_COL:
-				return true;
-
 			default:
 				return false;
 		}
@@ -436,56 +589,6 @@ public class LayerElement extends Element {
 				}
 				return nameEditor;
 
-			case LayerCompositionView.MODE_COL:
-				if (modeEditor == null) {
-					modeEditor = new ComboBoxViewerCellEditor(viewer.getTree(), SWT.READ_ONLY);
-					modeEditor.setContenProvider(ArrayContentProvider.getInstance());
-					modeEditor.setLabelProvider(new BlendModeLabelProvider());
-					modeEditor.setActivationStyle(ComboBoxViewerCellEditor.DROP_DOWN_ON_PROGRAMMATIC_ACTIVATION);
-
-					Combo combo = (Combo) modeEditor.getControl();
-					combo.setVisibleItemCount(35);
-					combo.addSelectionListener(new SelectionAdapter() {
-						public void widgetSelected(SelectionEvent e) {
-							setCellEditorValue(LayerCompositionView.MODE_COL, modeEditor.getValue());
-						}
-					});
-				}
-				modeEditor.setInput(layerBlendModes);
-				return modeEditor;
-
-			case LayerCompositionView.TRACKMATTE_COL:
-				if (trackMatteEditor == null) {
-					trackMatteEditor = new ComboBoxViewerCellEditor(viewer.getTree(), SWT.READ_ONLY);
-					trackMatteEditor.setContenProvider(ArrayContentProvider.getInstance());
-					trackMatteEditor.setLabelProvider(new TrackMatteLabelProvider());
-					trackMatteEditor.setActivationStyle(ComboBoxViewerCellEditor.DROP_DOWN_ON_PROGRAMMATIC_ACTIVATION);
-
-					((Combo) trackMatteEditor.getControl()).addSelectionListener(new SelectionAdapter() {
-						public void widgetSelected(SelectionEvent e) {
-							setCellEditorValue(LayerCompositionView.TRACKMATTE_COL, trackMatteEditor.getValue());
-						}
-					});
-				}
-				trackMatteEditor.setInput(TrackMatte.values());
-				return trackMatteEditor;
-
-			case LayerCompositionView.PARENT_COL:
-				if (parentEditor == null) {
-					parentEditor = new ComboBoxViewerCellEditor(viewer.getTree(), SWT.READ_ONLY);
-					parentEditor.setContenProvider(ArrayContentProvider.getInstance());
-					parentEditor.setLabelProvider(new ParentLabelProvider());
-					parentEditor.setActivationStyle(ComboBoxViewerCellEditor.DROP_DOWN_ON_PROGRAMMATIC_ACTIVATION);
-
-					((Combo) parentEditor.getControl()).addSelectionListener(new SelectionAdapter() {
-						public void widgetSelected(SelectionEvent e) {
-							setCellEditorValue(LayerCompositionView.PARENT_COL, parentEditor.getValue());
-						}
-					});
-				}
-				parentEditor.setInput(getLayersForParentEditor());
-				return parentEditor;
-
 			default:
 				return null;
 		}
@@ -496,16 +599,6 @@ public class LayerElement extends Element {
 			case LayerCompositionView.NAME_COL:
 				return layer.getName();
 
-			case LayerCompositionView.MODE_COL:
-				return ((MediaLayer) layer).getBlendMode();
-
-			case LayerCompositionView.TRACKMATTE_COL:
-				return ((MediaLayer) layer).getTrackMatte();
-
-			case LayerCompositionView.PARENT_COL:
-				Layer parent = composition.getParentLayer(layer);
-				return (parent != null) ? parent : NO_PARENT;
-
 			default:
 				return null;
 		}
@@ -525,80 +618,12 @@ public class LayerElement extends Element {
 						if (layer instanceof TextLayer) newName = "テキスト";
 						else if (layer instanceof NullLayer) newName = "ヌル";
 						else if (layer instanceof CameraLayer) newName = "カメラ";
+						else if (layer instanceof LightLayer) newName = "ライト";
 					}
 				}
 				pm.postOperation(new RenameLayerOperation(pm, layer, newName));
 				break;
 			}
-
-			case LayerCompositionView.MODE_COL:
-				pm.postOperation(new ModifyLayerBlendModeOperation(pm, (MediaLayer) layer, (BlendMode) value));
-				break;
-
-			case LayerCompositionView.TRACKMATTE_COL:
-				pm.postOperation(new ModifyLayerTrackMatteOperation(pm, (MediaLayer) layer, (TrackMatte) value));
-				break;
-
-			case LayerCompositionView.PARENT_COL:
-				pm.postOperation(new ModifyLayerParentOperation(pm, layer, (value != NO_PARENT) ? (Layer) value : null));
-				break;
-		}
-	}
-
-	private static class BlendModeLabelProvider extends LabelProvider {
-		public String getText(Object element) {
-			return EnumLabels.getLabel((BlendMode) element);
-		}
-	}
-
-	private class TrackMatteLabelProvider extends LabelProvider {
-		public String getText(Object element) {
-			TrackMatte trackMatte = (TrackMatte) element;
-			if (trackMatte == TrackMatte.NONE) {
-				return EnumLabels.getLabel(trackMatte);
-			} else {
-				String name;
-				MediaLayer trackMatteLayer = getTrackMatteLayer();
-				if (!(trackMatteLayer instanceof ItemLayer<?>) || isLayerNameMode()) {
-					name = trackMatteLayer.getName();
-				} else {
-					name = ((ItemLayer<?>) trackMatteLayer).getItem().getName();
-				}
-				return String.format("%s - %s", EnumLabels.getLabel(trackMatte), name);
-			}
-		}
-	}
-
-	private Object[] getLayersForParentEditor() {
-		List<Object> layers = Util.newList();
-
-		for (Layer l : composition.getLayers()) {
-			Layer tmp = l;
-			while (tmp != null && tmp != layer) {
-				tmp = composition.getParentLayer(tmp);
-			}
-			if (tmp == null) {
-				layers.add(l);
-			}
-		}
-
-		layers.add(NO_PARENT);
-		Collections.reverse(layers);
-
-		return layers.toArray();
-	}
-
-	private static final Object NO_PARENT = new Object();
-
-	private class ParentLabelProvider extends LabelProvider {
-		public String getText(Object element) {
-			if (element == NO_PARENT) {
-				return "なし";
-			} else if (!(element instanceof ItemLayer<?>) || isLayerNameMode()) {
-				return ((Layer) element).getName();
-			} else {
-				return ((ItemLayer<?>) element).getItem().getName();
-			}
 		}
 	}
 
