@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -37,7 +37,7 @@ import ch.kuramo.javie.core.LayerNature;
 import ch.kuramo.javie.core.MediaLayer;
 import ch.kuramo.javie.core.TimeCode;
 
-public class TimeRemapElement extends LayerAnimatableValueElement {
+public class TimeRemapElement extends LayerAnimatableValueElement.ForSimpleProperty {
 
 	public TimeRemapElement(LayerElement parent) {
 		super(parent, parent.layer, "timeRemap", "タイムリマップ");
@@ -45,7 +45,7 @@ public class TimeRemapElement extends LayerAnimatableValueElement {
 
 	@Override
 	protected AnimatableValueElementDelegate<?> createDelegate() {
-		AnimatableDouble avalue = PropertyUtil.getProperty(layer, property);
+		AnimatableDouble avalue = PropertyUtil.getProperty(layer, getProperty());
 		return new TimeRemapElementDelegate(this, name, avalue, (MediaLayer) layer);
 	}
 
