@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -671,6 +671,7 @@ public class TimelineManager {
 			keyframeSelection.remove(kf);
 		}
 
+		view.updateDeleteActionState();
 		redraw();
 
 
@@ -690,10 +691,12 @@ public class TimelineManager {
 
 	public void clearKeyframeSelection() {
 		keyframeSelection.clear();
+		view.updateDeleteActionState();
 	}
 
 	public void addKeyframeSelection(Keyframe<?> keyframe, AnimatableValueElement element) {
 		keyframeSelection.put(keyframe, element);
+		view.updateDeleteActionState();
 	}
 
 	public boolean hasKeyframeSelection() {
@@ -714,7 +717,7 @@ public class TimelineManager {
 
 			if (element instanceof LayerAnimatableValueElement) {
 				LayerAnimatableValueElement avalueElem = (LayerAnimatableValueElement) element;
-				data[i++] = new Object[] { avalueElem.layer.getId(), -1, avalueElem.property, kf };
+				data[i++] = new Object[] { avalueElem.layer.getId(), -1, avalueElem.getProperty(), kf };
 			} else {
 				EffectAnimatableValueElement avalueElem = (EffectAnimatableValueElement) element;
 				data[i++] = new Object[] { avalueElem.layer.getId(),
