@@ -70,10 +70,10 @@ public abstract class AbstractAnimatableValue<V> implements AnimatableValue<V> {
 		_defaultValue = src._defaultValue;
 	}
 
-	public V value(RenderContext renderContext) {
-		if (_expression != null && !renderContext.checkCyclicEvaluation(this)) {
+	public V value(CoreContext context) {
+		if (_expression != null && !context.checkCyclicEvaluation(this)) {
 			try {
-				V value = renderContext.evaluateExpression(this);
+				V value = context.evaluateExpression(this);
 				if (value != null) {
 					return value;
 				}
@@ -84,16 +84,16 @@ public abstract class AbstractAnimatableValue<V> implements AnimatableValue<V> {
 				e.printStackTrace();
 			}
 		}
-		return interpolate(renderContext.getTime());
+		return interpolate(context.getTime());
 	}
 
-	public V valueAtTime(final Time time, final RenderContext renderContext) {
-		if (_expression != null && !renderContext.checkCyclicEvaluation(this)) {
+	public V valueAtTime(final Time time, final CoreContext context) {
+		if (_expression != null && !context.checkCyclicEvaluation(this)) {
 			try {
-				V value = renderContext.saveAndExecute(new WrappedOperation<V>() {
+				V value = context.saveAndExecute(new WrappedOperation<V>() {
 					public V execute() {
-						renderContext.setTime(time);
-						return renderContext.evaluateExpression(AbstractAnimatableValue.this);
+						context.setTime(time);
+						return context.evaluateExpression(AbstractAnimatableValue.this);
 					}
 				});
 				if (value != null) {
@@ -110,30 +110,29 @@ public abstract class AbstractAnimatableValue<V> implements AnimatableValue<V> {
 	}
 
 	public void values(
-			IObjectArray<V> values, Time sampleDuration, int evalResolution,
-			RenderContext renderContext) {
+			IObjectArray<V> values, Time sampleDuration, int animationRate,
+			CoreContext context) {
 
-		valuesAtTime(values, sampleDuration, evalResolution,
-				renderContext.getTime(), renderContext);
+		valuesAtTime(values, sampleDuration, animationRate, context.getTime(), context);
 	}
 
 	public void valuesAtTime(
-			final IObjectArray<V> values, final Time sampleDuration, final int evalResolution,
-			final Time time, final RenderContext renderContext) {
+			final IObjectArray<V> values, final Time sampleDuration, final int animationRate,
+			final Time time, final CoreContext context) {
 
 		final Object[] array = values.getArray();
 		final int len = values.getLength();
 
-		if (_expression != null && !renderContext.checkCyclicEvaluation(this)) {
+		if (_expression != null && !context.checkCyclicEvaluation(this)) {
 			try {
-				renderContext.saveAndExecute(new WrappedOperation<Object>() {
+				context.saveAndExecute(new WrappedOperation<Object>() {
 					public Object execute() {
 						V v = null;
 						for (int i = 0; i < len; ++i) {
-							if ((i % evalResolution) == 0) {
+							if ((i % animationRate) == 0) {
 								Time t = time.add(new Time(sampleDuration.timeValue*i, sampleDuration.timeScale));
-								renderContext.setTime(t);
-								v = renderContext.evaluateExpression(AbstractAnimatableValue.this);
+								context.setTime(t);
+								v = context.evaluateExpression(AbstractAnimatableValue.this);
 							}
 							array[i] = v;
 						}
@@ -150,7 +149,7 @@ public abstract class AbstractAnimatableValue<V> implements AnimatableValue<V> {
 
 		V v = null;
 		for (int i = 0; i < len; ++i) {
-			if ((i % evalResolution) == 0) {
+			if ((i % animationRate) == 0) {
 				Time t = time.add(new Time(sampleDuration.timeValue*i, sampleDuration.timeScale));
 				v = interpolate(t);
 			}
@@ -218,20 +217,17 @@ public abstract class AbstractAnimatableValue<V> implements AnimatableValue<V> {
 		return _keyframes.get(time);
 	}
 
+	/**
+	 * ProjectEncoderImplでエンコードする際に必要。APIの一部では無い。
+	 */
+	@Deprecated
 	public Collection<Keyframe<V>> getKeyframes() {
 		return Collections.unmodifiableCollection(_keyframes.values());
 	}
 
-	public Collection<Keyframe<V>> getKeyframes(Time fromTime, Time toTime) {
-		return Collections.unmodifiableCollection(_keyframes.subMap(fromTime, toTime).values());
-	}
-
-	public SortedMap<Time, Keyframe<V>> headKeyframeMap(Time toTime) {
-		return Collections.unmodifiableSortedMap(_keyframes.headMap(toTime));
-	}
-
-	public SortedMap<Time, Keyframe<V>> tailKeyframeMap(Time fromTime) {
-		return Collections.unmodifiableSortedMap(_keyframes.tailMap(fromTime));
+	@JSONHint(ignore=true)
+	public SortedMap<Time, Keyframe<V>> getKeyframeMap() {
+		return Collections.unmodifiableSortedMap(_keyframes);
 	}
 
 	public Keyframe<V> putKeyframe(Keyframe<V> keyframe) {
@@ -293,10 +289,10 @@ public abstract class AbstractAnimatableValue<V> implements AnimatableValue<V> {
 		return Interpolation.HOLD;
 	}
 
-	protected V valueAtTime(double t, RenderContext renderContext) {
-		int timeScale = renderContext.getTime().timeScale;
+	protected V valueAtTime(double t, CoreContext context) {
+		int timeScale = context.getTime().timeScale;
 		Time time = new Time((long) (t * timeScale), timeScale); 
-		return valueAtTime(time, renderContext);
+		return valueAtTime(time, context);
 	}
 
 }
