@@ -23,7 +23,6 @@
 
 package ch.kuramo.javie.core;
 
-import net.arnx.jsonic.JSONHint;
 import ch.kuramo.javie.api.Time;
 
 public abstract class AbstractLayer implements Layer {
@@ -32,8 +31,6 @@ public abstract class AbstractLayer implements Layer {
 
 	private String _parentId;
 
-	private Layer _parent;
-
 	private String _name = "名称未設定レイヤー";
 
 	private boolean _shy;
@@ -71,17 +68,6 @@ public abstract class AbstractLayer implements Layer {
 		_parentId = parentId;
 	}
 
-	@JSONHint(ignore=true)
-	public Layer getParent() {
-		return _parent;
-	}
-
-	@JSONHint(ignore=true)
-	public void setParent(Layer parent) {
-		_parentId = (parent != null) ? parent.getId() : null;
-		_parent = parent;
-	}
-
 	public String getName() {
 		return _name;
 	}
@@ -131,13 +117,7 @@ public abstract class AbstractLayer implements Layer {
 	}
 
 	public void afterDecode(Project p, LayerComposition c) throws ProjectDecodeException {
-		if (_parentId != null) {
-			_parent = c.getLayer(_parentId);
-			if (_parent == null) {
-				throw new ProjectDecodeException(
-						"no such parent layer found: id=" + _parentId);
-			}
-		}
+		// nothing to do
 	}
 
 	public void prepareExpression(ExpressionScope scope) {
@@ -146,14 +126,14 @@ public abstract class AbstractLayer implements Layer {
 
 	public class LayerExpressionElement {
 
-		protected final RenderContext renderContext;
+		protected final CoreContext context;
 
-		public LayerExpressionElement(RenderContext renderContext) {
-			this.renderContext = renderContext;
+		public LayerExpressionElement(CoreContext context) {
+			this.context = context;
 		}
 
 		protected <T> T elem(Expressionee exprnee) {
-			return renderContext.getExpressionElement(exprnee);
+			return context.getExpressionElement(exprnee);
 		}
 
 		public LayerExpressionElement getThisLayer() {
@@ -164,6 +144,11 @@ public abstract class AbstractLayer implements Layer {
 		public double getInPoint()		{ return _inPoint.toSecond(); }
 		public double getOutPoint()		{ return _outPoint.toSecond(); }
 		public double getRate()			{ return _rate; }
+
+		@Override
+		public String toString() {
+			return _id;
+		}
 	}
 
 }
