@@ -93,7 +93,7 @@ public class AnimatableColor extends ArithmeticalAnimatableValue<Color> implemen
 
 	@Override
 	protected double[] toArray(Color value) {
-		return new double[] { value.r, value.g, value.b, value.a };
+		return new double[] { value.rawRed, value.rawGreen, value.rawBlue, value.rawAlpha };
 	}
 
 	@Override
