@@ -67,12 +67,12 @@ public class AnimatableDouble extends ArithmeticalAnimatableValue<Double> implem
 		return d[0];
 	}
 
-	public Scriptable createExpressionElement(final RenderContext renderContext) {
+	public Scriptable createExpressionElement(final CoreContext context) {
 		DoubleProperty dp = new DoubleProperty() {
-			public double getValue()			{ return value(renderContext); }
-			public double valueAtTime(double t)	{ return AnimatableDouble.this.valueAtTime(t, renderContext); }
+			public double getValue()			{ return value(context); }
+			public double valueAtTime(double t)	{ return AnimatableDouble.this.valueAtTime(t, context); }
 		};
-		return renderContext.toNativeJavaObject(dp, ScriptRuntime.NumberClass);
+		return context.toNativeJavaObject(dp, ScriptRuntime.NumberClass);
 	}
 
 }
