@@ -67,12 +67,12 @@ public class AnimatableInteger extends ArithmeticalAnimatableValue<Integer> impl
 		return Integer.valueOf(Math.round((float) d[0]));
 	}
 
-	public Scriptable createExpressionElement(final RenderContext renderContext) {
+	public Scriptable createExpressionElement(final CoreContext context) {
 		IntegerProperty ip = new IntegerProperty() {
-			public int getValue()				{ return value(renderContext); }
-			public int valueAtTime(double t)	{ return AnimatableInteger.this.valueAtTime(t, renderContext); }
+			public int getValue()				{ return value(context); }
+			public int valueAtTime(double t)	{ return AnimatableInteger.this.valueAtTime(t, context); }
 		};
-		return renderContext.toNativeJavaObject(ip, ScriptRuntime.NumberClass);
+		return context.toNativeJavaObject(ip, ScriptRuntime.NumberClass);
 	}
 
 }
