@@ -23,7 +23,6 @@
 
 package ch.kuramo.javie.core;
 
-import java.util.Collection;
 import java.util.Set;
 import java.util.SortedMap;
 
@@ -32,17 +31,17 @@ import ch.kuramo.javie.api.Time;
 
 public interface AnimatableValue<V> extends Expressioner<V>, Expressionee {
 
-	V value(RenderContext renderContext);
+	V value(CoreContext context);
 
-	V valueAtTime(Time time, RenderContext renderContext);
+	V valueAtTime(Time time, CoreContext context);
 
 	void values(
-			IObjectArray<V> values, Time sampleDuration, int evalResolution,
-			RenderContext renderContext);
+			IObjectArray<V> values, Time sampleDuration, int animationRate,
+			CoreContext context);
 
 	void valuesAtTime(
-			IObjectArray<V> values, Time sampleDuration, int evalResolution,
-			Time time, RenderContext renderContext);
+			IObjectArray<V> values, Time sampleDuration, int animationRate,
+			Time time, CoreContext context);
 
 	V getStaticValue();
 
@@ -50,13 +49,7 @@ public interface AnimatableValue<V> extends Expressioner<V>, Expressionee {
 
 	Keyframe<V> getKeyframe(Time time);
 
-	Collection<Keyframe<V>> getKeyframes();
-
-	Collection<Keyframe<V>> getKeyframes(Time fromTime, Time toTime);
-
-	SortedMap<Time, Keyframe<V>> headKeyframeMap(Time toTime);
-
-	SortedMap<Time, Keyframe<V>> tailKeyframeMap(Time fromTime);
+	SortedMap<Time, Keyframe<V>> getKeyframeMap();
 
 	Keyframe<V> putKeyframe(Keyframe<V> keyframe);
 
