@@ -25,12 +25,8 @@ package ch.kuramo.javie.core;
 
 import java.io.File;
 
-import ch.kuramo.javie.api.Time;
-
 public interface ImageSequenceItem extends MediaItem/*, FileItem*/ {
 
 	File getFile();
 
-	Time getFrameDuration();
-
 }
