@@ -68,6 +68,8 @@ public class WindowsDirectShowSource implements MediaSource {
 
 	private static final int FOURCC_CUVC = toFourCC("CUVC");
 
+	private static final int FOURCC_ULRA = toFourCC("ULRA");
+
 	private static int toFourCC(String fourCCStr) {
 		if (fourCCStr.length() != 4) {
 			throw new IllegalArgumentException(fourCCStr);
@@ -328,6 +330,9 @@ public class WindowsDirectShowSource implements MediaSource {
 						break;
 
 					case STRAIGHT:
+						gl.glClearColor(0, 0, 0, 0);
+						gl.glClear(GL2.GL_COLOR_BUFFER_BIT);
+
 						gl.glEnable(GL2.GL_BLEND);
 						gl.glBlendFuncSeparate(GL2.GL_SRC_ALPHA, GL2.GL_ZERO, GL2.GL_ONE, GL2.GL_ZERO);
 						gl.glBlendEquation(GL2.GL_FUNC_ADD);
@@ -349,20 +354,16 @@ public class WindowsDirectShowSource implements MediaSource {
 				}
 
 				// テクスチャは useFramebuffer によってすでにバインドされている。
-				//gl.glBindTexture(GL2.GL_TEXTURE_RECTANGLE, srcTexture);
-				//gl.glEnable(GL2.GL_TEXTURE_RECTANGLE);
+				//gl.glBindTexture(GL2.GL_TEXTURE_2D, srcTexture);
+				//gl.glEnable(GL2.GL_TEXTURE_2D);
 
 				gl.glColor4f(1, 1, 1, 1);
 
 				gl.glBegin(GL2.GL_QUADS);
-				gl.glTexCoord2f(0, 0);
-				gl.glVertex2f(0, 0);
-				gl.glTexCoord2f(srcBounds.width, 0);
-				gl.glVertex2f(srcBounds.width, 0);
-				gl.glTexCoord2f(srcBounds.width, srcBounds.height);
-				gl.glVertex2f(srcBounds.width, srcBounds.height);
-				gl.glTexCoord2f(0, srcBounds.height);
-				gl.glVertex2f(0, srcBounds.height);
+				gl.glTexCoord2f(0, 0); gl.glVertex2f(0, 0);
+				gl.glTexCoord2f(1, 0); gl.glVertex2f(srcBounds.width, 0);
+				gl.glTexCoord2f(1, 1); gl.glVertex2f(srcBounds.width, srcBounds.height);
+				gl.glTexCoord2f(0, 1); gl.glVertex2f(0, srcBounds.height);
 				gl.glEnd();
 			}
 		};
@@ -540,9 +541,13 @@ public class WindowsDirectShowSource implements MediaSource {
 		if (options == null) {
 			options = new MediaOptions(availableOptions);
 
-			if (isVideoAvailable() && _fourCC == FOURCC_CUVC) {
-				options.setAlphaType(AlphaType.IGNORE);
-				options.setFlipVertical(true);
+			if (isVideoAvailable()) {
+				if (_fourCC != FOURCC_ULRA) {
+					options.setAlphaType(AlphaType.IGNORE);
+				}
+				if (_fourCC == FOURCC_CUVC) {
+					options.setFlipVertical(true);
+				}
 			}
 
 		} else {
