@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010 Yoshikazu Kuramochi
+ * Copyright (c) 2010,2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -35,6 +35,9 @@ import com.google.inject.Inject;
 
 public class AntiAliasSupportImpl implements IAntiAliasSupport {
 
+	private static final boolean MACOSX = System.getProperty("os.name")
+					.toLowerCase().replaceAll("\\s+", "").contains("macosx");
+
 	private final RenderContext context;
 
 	private final int samples;
@@ -102,7 +105,7 @@ public class AntiAliasSupportImpl implements IAntiAliasSupport {
 			renderBuffer[i] = rb[0];
 		}
 
-		if (internalFormat != this.internalFormat[i] || width > this.width[i] || height > this.height[i]) {
+		if (internalFormat != this.internalFormat[i] || width != this.width[i] || height != this.height[i]) {
 			gl.glBindRenderbuffer(GL2.GL_RENDERBUFFER, renderBuffer[i]);
 			gl.glRenderbufferStorageMultisample(GL2.GL_RENDERBUFFER, samples, internalFormat, width, height);
 			this.internalFormat[i] = internalFormat;
@@ -114,14 +117,20 @@ public class AntiAliasSupportImpl implements IAntiAliasSupport {
 	}
 
 	private int getColorRenderBuffer(GL2 gl, int width, int height) {
+		// TODO あまりに大きなサイズの場合の対策が必要。
+		//      例えば、過去数十回程度の使用履歴を記録しておいて、その中で一番大きなサイズまで縮小するとか、
+		//      もっと単純に、数十回使用する毎に小さなサイズに作りなおす等。
+		width = Math.max(width, this.width[0]);
+		height = Math.max(height, this.height[0]);
+
 		return getRenderBuffer(gl, 0, context.getColorMode().glInternalFormat, width, height);
 	}
 
-	private int getDepthRenderBuffer(GL2 gl, int width, int height) {
+	private int getDepthRenderBuffer(GL2 gl) {
 		// TODO 常時 GL_DEPTH_COMPONENT32 を指定した方が良いかも？
 		int internalFormat = (context.getColorMode() == ColorMode.RGBA32_FLOAT)
 								? GL2.GL_DEPTH_COMPONENT32 : GL2.GL_DEPTH_COMPONENT;
-		return getRenderBuffer(gl, 1, internalFormat, width, height);
+		return getRenderBuffer(gl, 1, internalFormat, width[0], height[0]);
 	}
 
 	private void clear(GL2 gl, boolean clearDepth, Color clearColor) {
@@ -176,8 +185,17 @@ public class AntiAliasSupportImpl implements IAntiAliasSupport {
 
 				if (depthTest) {
 					gl.glFramebufferRenderbuffer(GL2.GL_DRAW_FRAMEBUFFER, GL2.GL_DEPTH_ATTACHMENT,
-							GL2.GL_RENDERBUFFER, getDepthRenderBuffer(gl, width, height));
+							GL2.GL_RENDERBUFFER, getDepthRenderBuffer(gl));
 					gl.glEnable(GL2.GL_DEPTH_TEST);
+
+				} else if (MACOSX && renderBuffer[1] != 0) {
+					// Macの場合、一度デプスバッファ用のレンダーバッファをアタッチすると
+					// それ以降、カラーバッファ用のレンダーバッファとデプスバッファ用の
+					// レンダーバッファのサイズが常に一致していないと正常に動作しない。
+					// そのため、一度でもデプステストを使用した後はデプスバッファ用の
+					// レンダーバッファを常に設定する。
+					gl.glFramebufferRenderbuffer(GL2.GL_DRAW_FRAMEBUFFER, GL2.GL_DEPTH_ATTACHMENT,
+							GL2.GL_RENDERBUFFER, getDepthRenderBuffer(gl));
 				}
 
 				clear(gl, depthTest, clearColor);
@@ -188,10 +206,8 @@ public class AntiAliasSupportImpl implements IAntiAliasSupport {
 				gl.glBlitFramebuffer(0, 0, width, height, 0, 0, width, height, GL2.GL_COLOR_BUFFER_BIT, GL2.GL_NEAREST);
 
 			} finally {
+				gl.glFramebufferRenderbuffer(GL2.GL_DRAW_FRAMEBUFFER, GL2.GL_DEPTH_ATTACHMENT, GL2.GL_RENDERBUFFER, 0);
 				gl.glFramebufferRenderbuffer(GL2.GL_DRAW_FRAMEBUFFER, GL2.GL_COLOR_ATTACHMENT0, GL2.GL_RENDERBUFFER, 0);
-				if (depthTest) {
-					gl.glFramebufferRenderbuffer(GL2.GL_DRAW_FRAMEBUFFER, GL2.GL_DEPTH_ATTACHMENT, GL2.GL_RENDERBUFFER, 0);
-				}
 				gl.glPopAttrib();
 			}
 		} finally {
