@@ -23,6 +23,7 @@
 
 package ch.kuramo.javie.core.internal.services;
 
+import java.util.Set;
 import java.util.concurrent.locks.ReentrantLock;
 
 import javax.media.opengl.GL;
@@ -38,6 +39,7 @@ import ch.kuramo.javie.api.ColorMode;
 import ch.kuramo.javie.api.Resolution;
 import ch.kuramo.javie.api.Time;
 import ch.kuramo.javie.core.Camera;
+import ch.kuramo.javie.core.Light;
 import ch.kuramo.javie.core.WrappedOperation;
 import ch.kuramo.javie.core.services.GLGlobal;
 import ch.kuramo.javie.core.services.RenderContext;
@@ -72,6 +74,8 @@ public class RenderContextImpl extends AbstractCoreContext implements RenderCont
 
 	private Camera _camera;
 
+	private Set<Light> _lights;
+
 	private AudioMode _audioMode;
 
 	private int _audioAnimationRate;
@@ -145,6 +149,7 @@ public class RenderContextImpl extends AbstractCoreContext implements RenderCont
 		_colorMode = null;
 		_videoFrameDuration = null;
 		_camera = null;
+		_lights = null;
 
 		_audioMode = null;
 		_audioAnimationRate = 0;
@@ -160,6 +165,7 @@ public class RenderContextImpl extends AbstractCoreContext implements RenderCont
 		_colorMode = null;
 		_videoFrameDuration = null;
 		_camera = null;
+		_lights = null;
 
 		_audioMode = null;
 		_audioAnimationRate = 0;
@@ -170,6 +176,7 @@ public class RenderContextImpl extends AbstractCoreContext implements RenderCont
 		ColorMode colorMode = _colorMode;
 		Time videoFrameDuration = _videoFrameDuration;
 		Camera camera = _camera;
+		Set<Light> lights = _lights;
 		int audioFrameCount = _audioFrameCount;
 		try {
 			return super.saveAndExecute(operation);
@@ -177,6 +184,7 @@ public class RenderContextImpl extends AbstractCoreContext implements RenderCont
 			_colorMode = colorMode;
 			_videoFrameDuration = videoFrameDuration;
 			_camera = camera;
+			_lights = lights;
 			_audioFrameCount = audioFrameCount;
 		}
 	}
@@ -229,6 +237,14 @@ public class RenderContextImpl extends AbstractCoreContext implements RenderCont
 		_camera = camera;
 	}
 
+	public Set<Light> getLights() {
+		return _lights;
+	}
+
+	public void setLights(Set<Light> lights) {
+		_lights = lights;
+	}
+
 	public AudioMode getAudioMode() {
 		if (_audioMode == null) {
 			throw new IllegalStateException("audioMode is not set.");
