@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -43,9 +43,11 @@ import javax.media.opengl.glu.GLU;
 
 import org.scannotation.AnnotationDB;
 
+import ch.kuramo.javie.api.GeometryInputType;
+import ch.kuramo.javie.api.GeometryOutputType;
 import ch.kuramo.javie.api.IShaderProgram;
-import ch.kuramo.javie.api.IShaderSourceFactory;
 import ch.kuramo.javie.api.ShaderType;
+import ch.kuramo.javie.api.annotations.GeometryShaderParameters;
 import ch.kuramo.javie.api.annotations.ShaderSource;
 import ch.kuramo.javie.api.services.IShaderRegistry;
 import ch.kuramo.javie.core.JavieRuntimeException;
@@ -83,27 +85,19 @@ public class ShaderRegistryImpl implements ShaderRegistry, IShaderRegistry {
 		}
 	}
 
-	private ShaderEntry getEntry(IShaderSourceFactory sourceFactory) {
-		String name = sourceFactory.getName();
+	private ShaderEntry getOrCreateEntry(String name, ShaderType type, String[] attach, String[] source,
+							GeometryInputType inputType, GeometryOutputType outputType, int verticesOut) {
 		synchronized (_entries) {
 			ShaderEntry entry = getEntry(name);
 			if (entry == null) {
-				String[] attach = null;
-				if (sourceFactory.isProgram()) {
-					attach = sourceFactory.getAttach();
-					if (attach == null) {
-						attach = new String[0];
-					}
-				}
-
-				String[] source = sourceFactory.getSource();
+				source = source.clone();
 				for (int i = 0; i < source.length; ++i) {
 					if (!source[i].endsWith("\n")) {
 						source[i] += "\n";
 					}
 				}
 
-				entry = new ShaderEntry(name, sourceFactory.getType(), attach, source);
+				entry = new ShaderEntry(name, type, attach, source, inputType, outputType, verticesOut);
 				try {
 					putEntry(name, entry);
 				} catch (ShaderRegistryException e) {
@@ -131,15 +125,42 @@ public class ShaderRegistryImpl implements ShaderRegistry, IShaderRegistry {
 		return getProgram(fqfn(clazz, name));
 	}
 
-	public IShaderProgram getProgram(IShaderSourceFactory sourceFactory) {
-		if (!sourceFactory.isProgram()) {
-			throw new IllegalArgumentException("must be program");
+	public IShaderProgram registerProgram(String name, ShaderType type, String[] attach, String[] source) {
+		if (attach == null) {
+			attach = new String[0];
 		}
-		return getEntry(sourceFactory).shaderProgram;
+		return getOrCreateEntry(name, type, attach, source, null, null, 0).shaderProgram;
 	}
 
-	public void addShader(IShaderSourceFactory sourceFactory) {
-		getEntry(sourceFactory);
+	public IShaderProgram registerProgramWithGeometryShader(
+			String name, String[] attach, String[] source,
+			GeometryInputType inputType, GeometryOutputType outputType, int verticesOut) {
+
+		if (inputType == null || outputType == null || verticesOut <= 0) {
+			throw new IllegalArgumentException();
+		}
+		if (attach == null) {
+			attach = new String[0];
+		}
+		return getOrCreateEntry(name, ShaderType.GEOMETRY_SHADER, attach, source,
+								inputType, outputType, verticesOut).shaderProgram;
+	}
+
+	public boolean isShaderRegistered(String name) {
+		return (getEntry(name) != null);
+	}
+
+	public void registerShader(String name, ShaderType type, String[] source) {
+		getOrCreateEntry(name, type, null, source, null, null, 0);
+	}
+
+	public void registerGeometryShader(String name, String[] source,
+			GeometryInputType inputType, GeometryOutputType outputType, int verticesOut) {
+
+		if (inputType == null || outputType == null || verticesOut <= 0) {
+			throw new IllegalArgumentException();
+		}
+		getOrCreateEntry(name, ShaderType.GEOMETRY_SHADER, null, source, inputType, outputType, verticesOut);
 	}
 
 	public void scanShaders(AnnotationDB db, ClassLoader cl) throws ShaderRegistryException {
@@ -159,9 +180,12 @@ public class ShaderRegistryImpl implements ShaderRegistry, IShaderRegistry {
 
 			for (Field field : clazz.getFields()) {
 				ShaderSource sourceAnno = null;
+				GeometryShaderParameters gspAnno = null;
 				for (Annotation fieldAnno : field.getAnnotations()) {
 					if (fieldAnno instanceof ShaderSource) {
 						sourceAnno = (ShaderSource) fieldAnno;
+					} else if (fieldAnno instanceof GeometryShaderParameters) {
+						gspAnno = (GeometryShaderParameters) fieldAnno;
 					}
 				}
 
@@ -198,7 +222,19 @@ public class ShaderRegistryImpl implements ShaderRegistry, IShaderRegistry {
 				}
 
 				String name = fqfn(field);
-				ShaderEntry entry = new ShaderEntry(name, sourceAnno.type(), attach, source);
+				ShaderEntry entry;
+
+				if (sourceAnno.type() == ShaderType.GEOMETRY_SHADER && gspAnno != null) {
+					if (gspAnno.verticesOut() <= 0) {
+						throw new ShaderRegistryException(fqfn(field), "'verticesOut' must be greater than 0");
+					}
+					entry = new ShaderEntry(name, sourceAnno.type(), attach, source,
+									gspAnno.inputType(), gspAnno.outputType(), gspAnno.verticesOut());
+
+				} else {
+					entry = new ShaderEntry(name, sourceAnno.type(), attach, source, null, null, 0);
+				}
+
 				putEntry(name, entry);
 			}
 		}
@@ -272,6 +308,9 @@ public class ShaderRegistryImpl implements ShaderRegistry, IShaderRegistry {
 			case VERTEX_SHADER:
 				type = GL2.GL_VERTEX_SHADER;
 				break;
+			case GEOMETRY_SHADER:
+				type = GL2.GL_GEOMETRY_SHADER_ARB;
+				break;
 			case FRAGMENT_SHADER:
 				type = GL2.GL_FRAGMENT_SHADER;
 				break;
@@ -308,6 +347,18 @@ public class ShaderRegistryImpl implements ShaderRegistry, IShaderRegistry {
 				}
 
 				gl.glAttachShader(program, attachEntry.shader);
+
+				if (attachEntry.shaderType == ShaderType.GEOMETRY_SHADER && attachEntry.geometryVerticesOut > 0) {
+					gl.glProgramParameteri(program, GL2.GL_GEOMETRY_INPUT_TYPE_ARB, attachEntry.geometryInputType.glInputType);
+					gl.glProgramParameteri(program, GL2.GL_GEOMETRY_OUTPUT_TYPE_ARB, attachEntry.geometryOutputType.glOutputType);
+					gl.glProgramParameteri(program, GL2.GL_GEOMETRY_VERTICES_OUT_ARB, attachEntry.geometryVerticesOut);
+				}
+			}
+
+			if (entry.shaderType == ShaderType.GEOMETRY_SHADER && entry.geometryVerticesOut > 0) {
+				gl.glProgramParameteri(program, GL2.GL_GEOMETRY_INPUT_TYPE_ARB, entry.geometryInputType.glInputType);
+				gl.glProgramParameteri(program, GL2.GL_GEOMETRY_OUTPUT_TYPE_ARB, entry.geometryOutputType.glOutputType);
+				gl.glProgramParameteri(program, GL2.GL_GEOMETRY_VERTICES_OUT_ARB, entry.geometryVerticesOut);
 			}
 
 			gl.glLinkProgram(program);
@@ -457,17 +508,25 @@ public class ShaderRegistryImpl implements ShaderRegistry, IShaderRegistry {
 		private final ShaderType shaderType;
 		private String[] attach;
 		private String[] source;
+		private final GeometryInputType geometryInputType;
+		private final GeometryOutputType geometryOutputType;
+		private final int geometryVerticesOut;
 		private volatile int shader;
 		private volatile int program;
 		private Map<String, Attribute> attributes;
 		private Map<String, Uniform> uniforms;
 		private final IShaderProgram shaderProgram;
 
-		private ShaderEntry(String name, ShaderType shaderType, String[] attach, String[] source) {
+		private ShaderEntry(
+				String name, ShaderType shaderType, String[] attach, String[] source,
+				GeometryInputType geometryInputType, GeometryOutputType geometryOutputType, int geometryVerticesOut) {
 			this.name = name;
 			this.shaderType = shaderType;
 			this.attach = attach;
 			this.source = source;
+			this.geometryInputType = geometryInputType;
+			this.geometryOutputType = geometryOutputType;
+			this.geometryVerticesOut = geometryVerticesOut;
 			this.shaderProgram = (attach != null) ? createShaderProgramImpl() : null;
 		}
 
