@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,9 +23,10 @@
 
 package ch.kuramo.javie.core.services;
 
+import java.util.Set;
 import java.util.concurrent.locks.ReentrantLock;
 
-import javax.media.opengl.GLCapabilities;
+import javax.media.opengl.GLCapabilitiesImmutable;
 import javax.media.opengl.GLContext;
 import javax.media.opengl.awt.GLCanvas;
 
@@ -38,7 +39,7 @@ public interface GLGlobal {
 
 	GLContext getSharedContext();
 
-	String getVendor();
+	boolean isMacOSX();
 
 	boolean isIntel();
 
@@ -46,10 +47,16 @@ public interface GLGlobal {
 
 	boolean isNvidia();
 
+	String getDriverVersion();
+
 	int getMaxSamples();
 
 	int getMaxTextureImageUnits();
 
+	Set<String> getExtensions();
+
+	String getReport();
+
 
 	void enterPoolMode(int poolSize);
 
@@ -58,9 +65,9 @@ public interface GLGlobal {
 	void destroyContext(GLContext context);
 
 
-	GLCapabilities getCanvasCapabilities();
+	GLCapabilitiesImmutable getCanvasCapabilities();
 
-	void setCanvasCapabilities(GLCapabilities capabilities);
+	void setCanvasCapabilities(GLCapabilitiesImmutable capabilities);
 
 	GLCanvas createCanvas();
 
