@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009,2010 Yoshikazu Kuramochi
+ * Copyright (c) 2009-2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -28,14 +28,17 @@ import java.util.List;
 import ch.kuramo.javie.api.IVideoBuffer;
 import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.core.Effect;
+import ch.kuramo.javie.core.MediaLayer;
 import ch.kuramo.javie.core.WrappedOperation;
 
 public interface VideoEffectPipeline {
 
-	VideoBounds getVideoBounds(List<Effect> effects,
+	VideoBounds getVideoBounds(
+			MediaLayer layer, List<Effect> effects,
 			WrappedOperation<VideoBounds> inputBoundsOperation);
 
-	IVideoBuffer doVideoEffects(List<Effect> effects,
+	IVideoBuffer doVideoEffects(
+			MediaLayer layer, List<Effect> effects,
 			WrappedOperation<VideoBounds> inputBoundsOperation,
 			WrappedOperation<IVideoBuffer> inputBufferOperation);
 
