@@ -91,6 +91,34 @@
 		return FALSE;
 	}
 	
+	if (mMediaType == MPEGMediaType) {
+		MediaHandler mpegMediaHandler = GetMediaHandler(media);
+		error = GetMoviesError();
+		if (error != noErr) {
+			NSLog(@"GetMediaHandler: error %d", error);
+			return FALSE;
+		}
+		
+		MHInfoEncodedFrameRateRecord encodedFrameRate;
+		Size encodedFrameRateSize = sizeof(encodedFrameRate);
+		
+		MoviesTask(mMovie, 0);
+		
+		error = MediaGetPublicInfo(mpegMediaHandler, kMHInfoEncodedFrameRate,
+								   &encodedFrameRate, &encodedFrameRateSize);
+		if (error != noErr) {
+			NSLog(@"MediaGetPublicInfo: error %d", error);
+			return FALSE;
+		}
+		
+		mSampleCount = FixedToFloat(encodedFrameRate.encodedFrameRate) * mDuration / mTimeScale;
+		
+	} else if (mMediaType == kQTQuartzComposerMediaType) {
+		// デュレーションは、JNI_OnLoad内でQuartzComposerDefaultMovieDurationを300*60*60秒に設定してある
+		mDuration = mTimeScale * 300*60*60;
+		mSampleCount = 0;
+	}
+	
 	return TRUE;
 }
 
@@ -221,21 +249,6 @@
 	return self;
 }
 
--(BOOL) getMediaInfo:(Track)track
-{
-	if (![super getMediaInfo:track]) {
-		return FALSE;
-	}
-	
-	if (mMediaType == kQTQuartzComposerMediaType) {
-		// デュレーションは、JNI_OnLoad内でQuartzComposerDefaultMovieDurationを300*60*60秒に設定してある
-		mDuration = mTimeScale * 300*60*60;
-		mSampleCount = 0;
-	}
-	
-	return TRUE;
-}
-
 -(void) setGraphicsMode
 {
 //	int bottomLayer = 0x80000000;
@@ -348,7 +361,7 @@
 	
 	glPushAttrib(GL_COLOR_BUFFER_BIT | GL_ENABLE_BIT | GL_TEXTURE_BIT | GL_CURRENT_BIT);
 	
-	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_EXT, texture, 0);
+	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, texture, 0);
 	glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT);
 	
 	switch (alphaType) {
@@ -364,6 +377,9 @@
 			
 		// AlphaType.STRAIGHT
 		case 1:
+			glClearColor(0, 0, 0, 0);
+			glClear(GL_COLOR_BUFFER_BIT);
+
 			glEnable(GL_BLEND);
 			glBlendFuncSeparate(GL_SRC_ALPHA, GL_ZERO, GL_ONE, GL_ZERO);
 			glBlendEquation(GL_FUNC_ADD);
@@ -411,7 +427,7 @@
 	glEnd();
 	
 	glFinish();
-	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_EXT, 0, 0);
+	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, 0, 0);
 	
 	glPopAttrib();
 	
