@@ -43,6 +43,7 @@ import ch.kuramo.javie.api.annotations.Effect.Categories;
 import ch.kuramo.javie.api.services.IShaderRegistry;
 import ch.kuramo.javie.api.services.IVideoEffectContext;
 import ch.kuramo.javie.api.services.IVideoRenderSupport;
+import ch.kuramo.javie.effects.TexCoords;
 
 import com.google.inject.Inject;
 
@@ -102,7 +103,7 @@ public class ChromaKey {
 			return input;
 		}
 
-		Color keyColor = context.value(this.keyColor).clamp();
+		Color keyColor = context.value(this.keyColor);
 		double similarity = context.value(this.similarity) / 100 * 0.4;	// 0.4というのはPEとほぼ同じにするための係数
 		double blend = context.value(this.blend) / 100 * 0.4;
 		double threshold = context.value(this.threshold) / 100;
@@ -120,10 +121,6 @@ public class ChromaKey {
 		Set<IVideoBuffer> tmpBuffers = new HashSet<IVideoBuffer>();
 		tmpBuffers.add(input);
 		try {
-			IShaderProgram program1 = maskOnly ? maskOnlyProgram : keyingProgram;
-			IShaderProgram program2 = (smoothing == Smoothing.LOW) ? smoothingLowProgram
-									: (smoothing == Smoothing.HIGH) ? smoothingHighProgram : null;
-
 			Set<GLUniformData> uniforms = new HashSet<GLUniformData>();
 			uniforms.add(new GLUniformData("texture", 0));
 			uniforms.add(new GLUniformData("keyUV", 2, FloatBuffer.wrap(new float[] { (float)u, (float)v })));
@@ -138,26 +135,41 @@ public class ChromaKey {
 			keyedOut = context.createVideoBuffer(bounds);
 			tmpBuffers.add(keyedOut);
 
+			IShaderProgram program = maskOnly ? maskOnlyProgram : keyingProgram;
+
 			if (smoothing == Smoothing.NONE || maskOnly) {
-				support.useShaderProgram(program1, uniforms, keyedOut, input);
+				support.useShaderProgram(program, uniforms, keyedOut, input);
 
 			} else {
 				smoothMask = context.createVideoBuffer(bounds);
 				tmpBuffers.add(smoothMask);
 
-				support.useShaderProgramMRT(program1, uniforms, new IVideoBuffer[] { keyedOut, smoothMask }, input);
+				support.useShaderProgramMRT(program, uniforms, new IVideoBuffer[] { keyedOut, smoothMask }, input);
 			}
 
-			if (program2 == null) {
-				tmpBuffers.remove(keyedOut);
-				return keyedOut;
+			float[] neib;
+			switch (smoothing) {
+				case LOW:
+					program = smoothingLowProgram;
+					neib = TexCoords.neighboringOffset8(bounds.width, bounds.height);
+					break;
+
+				case HIGH:
+					program = smoothingHighProgram;
+					neib = TexCoords.neighboringOffset24(bounds.width, bounds.height);
+					break;
+
+				default:
+					tmpBuffers.remove(keyedOut);
+					return keyedOut;
 			}
 
 			uniforms.clear();
 			uniforms.add(new GLUniformData("tex0", 0));
 			uniforms.add(new GLUniformData("tex1", 1));
+			uniforms.add(new GLUniformData("neib[0]", 2, FloatBuffer.wrap(neib)));
 
-			support.useShaderProgram(program2, uniforms, input, keyedOut, smoothMask);
+			support.useShaderProgram(program, uniforms, input, keyedOut, smoothMask);
 
 			tmpBuffers.remove(input);
 			return input;
