@@ -211,10 +211,10 @@ public class Shatter {
 	@Property(value="100", min="0")
 	private IAnimatableDouble cameraZoom;
 
-	@Property(value="1", min="0")
+	@Property(value="10", min="0")
 	private IAnimatableDouble cameraNear;
 
-	@Property(value="1000", min="0")
+	@Property(value="100000", min="0")
 	private IAnimatableDouble cameraFar;
 
 
@@ -681,6 +681,7 @@ public class Shatter {
 
 		uniforms.add(new GLUniformData("texture", 0));
 		uniforms.add(new GLUniformData("texOffset", 2, toFloatBuffer(-bounds.x, -bounds.y)));
+		uniforms.add(new GLUniformData("texSize", 2, toFloatBuffer(bounds.width, bounds.height)));
 		uniforms.add(new GLUniformData("patOrigin", 2, toFloatBuffer(patOrigin.x, patOrigin.y)));
 		uniforms.add(new GLUniformData("patDirMat", 2, 2, toFloatBuffer(patDirCos, patDirSin, -patDirSin, patDirCos)));
 
@@ -846,6 +847,7 @@ public class Shatter {
 		"attribute vec2 fragCenter;",
 		"",
 		"uniform vec2 texOffset;",
+		"uniform vec2 texSize;",
 		"uniform vec2 patOrigin;",
 		"uniform mat2 patDirMat;",
 		"",
@@ -920,18 +922,17 @@ public class Shatter {
 		"	vec3 pos2 = pos1 + vec3(fc, 0.0) + v*t + t*t*gravity;",
 		"	gl_Position = gl_ModelViewProjectionMatrix * vec4(pos2, 1.0);",
 		"",
-		"	vec4 texPos = vec4(pos0, 0.0, 1.0);",
-		"	gl_TexCoord[0] = texPos + vec4(texOffset, 0.0, 0.0);",
+		"	gl_TexCoord[0] = vec4((pos0 + texOffset) / texSize, 0.0, 1.0);",
 		"}"
 	};
 
 	@ShaderSource(attach="vertex_shader")
 	public static final String[] SHATTER = {
-		"uniform sampler2DRect texture;",
+		"uniform sampler2D texture;",
 		"",
 		"void main(void)",
 		"{",
-		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
+		"	vec4 color = texture2D(texture, gl_TexCoord[0].st);",
 		"	if (color.a == 0.0) discard;",
 		"	gl_FragColor = color;",
 		"}"
