@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010 Yoshikazu Kuramochi
+ * Copyright (c) 2010,2011 Yoshikazu Kuramochi
  * All rights reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,6 +27,7 @@ import java.nio.FloatBuffer;
 import java.util.HashSet;
 import java.util.Set;
 
+import javax.media.opengl.GL2;
 import javax.media.opengl.GLUniformData;
 
 import ch.kuramo.javie.api.Color;
@@ -37,6 +38,7 @@ import ch.kuramo.javie.api.IAnimatableDouble;
 import ch.kuramo.javie.api.IAnimatableEnum;
 import ch.kuramo.javie.api.IShaderProgram;
 import ch.kuramo.javie.api.IVideoBuffer;
+import ch.kuramo.javie.api.Quality;
 import ch.kuramo.javie.api.Resolution;
 import ch.kuramo.javie.api.VideoBounds;
 import ch.kuramo.javie.api.annotations.Effect;
@@ -45,6 +47,7 @@ import ch.kuramo.javie.api.annotations.ShaderSource;
 import ch.kuramo.javie.api.annotations.Effect.Categories;
 import ch.kuramo.javie.api.services.IBlurSupport;
 import ch.kuramo.javie.api.services.IShaderRegistry;
+import ch.kuramo.javie.api.services.ITexture1DSupport;
 import ch.kuramo.javie.api.services.IVideoEffectContext;
 import ch.kuramo.javie.api.services.IVideoRenderSupport;
 import ch.kuramo.javie.api.services.IBlurSupport.BlurDimensions;
@@ -59,29 +62,35 @@ public class Border {
 	public enum Position { OUTSIDE, INSIDE, BOTH }
 
 
-	@Property(value="50", min="0", max="100")
+	@Property(value="3", min="0", max="50")
+	private IAnimatableDouble size;
+
+	@Property(value="0", min="0", max="100")
+	private IAnimatableDouble softness;
+
+	@Property
+	private IAnimatableBoolean borderOnly;
+
+	@Property(value="100", min="0", max="100")
 	private IAnimatableDouble threshold;
 
+	@Property(value="0", min="0", max="100")
+	private IAnimatableDouble cutoff;
+
 	@Property
 	private IAnimatableEnum<Style> style;
 
 	@Property
 	private IAnimatableEnum<Position> position;
 
-	@Property(value="3", min="0", max="50")
-	private IAnimatableDouble size;
-
-	@Property
-	private IAnimatableBoolean accuracy;
-
 	@Property("1,0,0")
 	private IAnimatableColor color;
 
 	@Property(value="100", min="0", max="100")
 	private IAnimatableDouble opacity;
 
-	@Property
-	private IAnimatableBoolean borderOnly;
+	@Property("true")
+	private IAnimatableBoolean accuracy;
 
 
 	private final IVideoEffectContext context;
@@ -90,7 +99,9 @@ public class Border {
 
 	private final IBlurSupport blurSupport;
 
-	private final IShaderProgram binarizationProgram;
+	private final ITexture1DSupport tex1DSupport;
+
+	private final IShaderProgram thresholdProgram;
 
 	private final IShaderProgram outsideProgram;
 
@@ -107,12 +118,13 @@ public class Border {
 	@Inject
 	public Border(
 			IVideoEffectContext context, IVideoRenderSupport support,
-			IBlurSupport blurSupport, IShaderRegistry shaders) {
+			IBlurSupport blurSupport, ITexture1DSupport tex1DSupport, IShaderRegistry shaders) {
 
 		this.context = context;
 		this.support = support;
 		this.blurSupport = blurSupport;
-		binarizationProgram = shaders.getProgram(Border.class, "BINARIZATION");
+		this.tex1DSupport = tex1DSupport;
+		thresholdProgram = shaders.getProgram(Border.class, "THRESHOLD");
 		outsideProgram = shaders.getProgram(Border.class, "OUTSIDE");
 		outsideBOProgram = shaders.getProgram(Border.class, "OUTSIDE_BORDER_ONLY");
 		insideProgram = shaders.getProgram(Border.class, "INSIDE");
@@ -153,9 +165,11 @@ public class Border {
 
 		Resolution resolution = context.getVideoResolution();
 		double threshold = context.value(this.threshold) / 100;
+		double cutoff = context.value(this.cutoff) / 100;
 		Style style = context.value(this.style);
 		Position position = context.value(this.position);
 		double size = resolution.scale(context.value(this.size));
+		double softness = context.value(this.softness) / 100;
 		boolean accuracy = context.value(this.accuracy);
 		Color color = context.value(this.color);
 		double opacity = context.value(this.opacity) / 100;
@@ -176,47 +190,75 @@ public class Border {
 				throw new RuntimeException("unknown Position: " + position);
 		}
 
-		IVideoBuffer binary = null;
+		IVideoBuffer thresholded = null;
 		IVideoBuffer blurred = null;
+		IVideoBuffer output = null;
 		try {
 			Set<GLUniformData> uniforms = new HashSet<GLUniformData>();
 			uniforms.add(new GLUniformData("texture", 0));
-			uniforms.add(new GLUniformData("threshold", (float)threshold));
+			uniforms.add(new GLUniformData("cutoff", (float)cutoff));
+			uniforms.add(new GLUniformData("t_minus_c", (float)(threshold-cutoff)));
 
-			if (accuracy && !context.getColorMode().isFloat()) {
-				binary = support.createVideoBuffer(source.getBounds(), ColorMode.RGBA16_FLOAT);
+			if (context.getColorMode().isFloat()
+					|| !accuracy || context.getQuality() == Quality.DRAFT || resolution.scale < 1) {
+				thresholded = support.createVideoBuffer(source.getBounds());
 			} else {
-				binary = support.createVideoBuffer(source.getBounds());
+				thresholded = support.createVideoBuffer(source.getBounds(), ColorMode.RGBA16_FLOAT);
 			}
 
-			support.useShaderProgram(binarizationProgram, uniforms, binary, source);
+			support.useShaderProgram(thresholdProgram, uniforms, thresholded, source);
 
 			if (size > 0) {
 				switch (style) {
 					case SMOOTH:
-						blurred = blurSupport.gaussianBlur(binary, size, BlurDimensions.BOTH, false, false);
+						blurred = blurSupport.gaussianBlur(thresholded, size, BlurDimensions.BOTH, false, false);
 						break;
 					case BOX:
-						blurred = blurSupport.boxBlur(binary, size, BlurDimensions.BOTH, false, false, 1);
+						blurred = blurSupport.boxBlur(thresholded, size, BlurDimensions.BOTH, false, false, 1);
 						break;
 					default:
 						throw new RuntimeException("unknown Style: " + style);
 				}
 			} else {
-				blurred = binary;
+				blurred = thresholded;
 			}
 
+			output = support.createVideoBuffer(blurred.getBounds());
+
+			final int asinTexture = getAsinTexture();
+			final IVideoBuffer outbuf = output;
+			final IVideoBuffer inbuf0 = blurred;
+			final IVideoBuffer inbuf1 = source;
+			Runnable operation = new Runnable() {
+				public void run() {
+					GL2 gl = context.getGL().getGL2();
+					gl.glActiveTexture(GL2.GL_TEXTURE2);
+					gl.glBindTexture(GL2.GL_TEXTURE_1D, asinTexture);
+					support.ortho2D(outbuf);
+					support.quad2D(outbuf, inbuf0, inbuf1);
+				}
+			};
+
 			uniforms.clear();
 			uniforms.add(new GLUniformData("texBlurred", 0));
 			uniforms.add(new GLUniformData("texSource", 1));
+			uniforms.add(new GLUniformData("asinTexture", 2));
 			uniforms.add(new GLUniformData("threshold", (float)threshold));
 			uniforms.add(new GLUniformData("size", (float)size));
+			uniforms.add(new GLUniformData("softness", (float)softness));
 			uniforms.add(new GLUniformData("color", 4, toFloatBuffer(color.r*opacity, color.g*opacity, color.b*opacity, opacity)));
-			return support.useShaderProgram(program, uniforms, null, blurred, source);
+
+			int pushAttribs = GL2.GL_TEXTURE_BIT;
+			support.useShaderProgram(program, uniforms, operation, pushAttribs, output, blurred, source);
+
+			IVideoBuffer result = output;
+			output = null;
+			return result;
 
 		} finally {
-			if (blurred != null && blurred != binary) blurred.dispose(); 
-			if (binary != null) binary.dispose();
+			if (blurred != null && blurred != thresholded) blurred.dispose(); 
+			if (thresholded != null) thresholded.dispose();
+			if (output != null) output.dispose();
 			source.dispose();
 		}
 	}
@@ -229,15 +271,37 @@ public class Border {
 		return FloatBuffer.wrap(array);
 	}
 
+
+	private static volatile int asinTexture;
+
+	private int getAsinTexture() {
+		if (asinTexture == 0) {
+			float[] array = new float[256];
+			for (int i = 0; i < array.length; ++i) {
+				double t = i/255.0;
+				array[i] = (float)(Math.asin(2*t-1)*2/Math.PI);
+			}
+
+			synchronized (Border.class) {
+				if (asinTexture == 0) {
+					asinTexture = tex1DSupport.texture1DFromArray(array, GL2.GL_ALPHA, GL2.GL_ALPHA32F);
+				}
+			}
+		}
+		return asinTexture;
+	}
+
+
 	@ShaderSource
-	public static final String[] BINARIZATION = {
+	public static final String[] THRESHOLD = {
 		"uniform sampler2D texture;",
-		"uniform float threshold;",
+		"uniform float cutoff;",
+		"uniform float t_minus_c;",
 		"",
 		"void main(void)",
 		"{",
 		"	float alpha = texture2D(texture, gl_TexCoord[0].st).a;",
-		"	gl_FragColor = (alpha <= threshold) ? vec4(0.0) : vec4(1.0);",
+		"	gl_FragColor = vec4(clamp(alpha-cutoff, 0.0, t_minus_c) / t_minus_c);",
 		"}"
 	};
 
@@ -245,20 +309,22 @@ public class Border {
 	public static final String[] OUTSIDE = {
 		"uniform sampler2D texBlurred;",
 		"uniform sampler2D texSource;",
+		"uniform sampler1D asinTexture;",
 		"uniform float threshold;",
 		"uniform float size;",
+		"uniform float softness;",
 		"uniform vec4 color;",
 		"",
 		"void main(void)",
 		"{",
 		"	vec4 source = texture2D(texSource, gl_TexCoord[1].st);",
-		"	if (source.a > threshold) {",
+		"	if (source.a >= threshold) {",
 		"		gl_FragColor = source + color*(1.0-source.a);",
 		"	} else {",
 		"		float a = texture2D(texBlurred, gl_TexCoord[0].st).a;",
 		"		if (a > 0.00621) {",
-		"			a = 1.0+asin(2.0*(a-0.00621)-1.0)*2.0/3.14159265358979323846264;",
-		"			a = clamp(a*size*(1.0-size*0.016), 0.0, 1.0);",
+		"			a = 1.0+texture1D(asinTexture, a-0.00621).a;",
+		"			a = clamp(a*size*(1.0-size*0.01), 0.0, 1.0) * smoothstep(0.0, softness, a);",
 		"			gl_FragColor = source + color*(1.0-source.a)*a;",
 		"		} else {",
 		"			gl_FragColor = source;",
@@ -271,20 +337,22 @@ public class Border {
 	public static final String[] OUTSIDE_BORDER_ONLY = {
 		"uniform sampler2D texBlurred;",
 		"uniform sampler2D texSource;",
+		"uniform sampler1D asinTexture;",
 		"uniform float threshold;",
 		"uniform float size;",
+		"uniform float softness;",
 		"uniform vec4 color;",
 		"",
 		"void main(void)",
 		"{",
 		"	vec4 source = texture2D(texSource, gl_TexCoord[1].st);",
-		"	if (source.a > threshold) {",
+		"	if (source.a >= threshold) {",
 		"		gl_FragColor = color*(1.0-source.a);",
 		"	} else {",
 		"		float a = texture2D(texBlurred, gl_TexCoord[0].st).a;",
 		"		if (a > 0.00621) {",
-		"			a = 1.0+asin(2.0*(a-0.00621)-1.0)*2.0/3.14159265358979323846264;",
-		"			a = clamp(a*size*(1.0-size*0.016), 0.0, 1.0);",
+		"			a = 1.0+texture1D(asinTexture, a-0.00621).a;",
+		"			a = clamp(a*size*(1.0-size*0.01), 0.0, 1.0) * smoothstep(0.0, softness, a);",
 		"			gl_FragColor = color*(1.0-source.a)*a;",
 		"		} else {",
 		"			gl_FragColor = vec4(0.0);",
@@ -297,20 +365,22 @@ public class Border {
 	public static final String[] INSIDE = {
 		"uniform sampler2D texBlurred;",
 		"uniform sampler2D texSource;",
+		"uniform sampler1D asinTexture;",
 		"uniform float threshold;",
 		"uniform float size;",
+		"uniform float softness;",
 		"uniform vec4 color;",
 		"",
 		"void main(void)",
 		"{",
 		"	vec4 source = texture2D(texSource, gl_TexCoord[1].st);",
-		"	if (source.a <= threshold) {",
+		"	if (source.a < threshold) {",
 		"		gl_FragColor = color*(source.a/threshold) + source*(1.0-color.a);",
 		"	} else {",
 		"		float a = texture2D(texBlurred, gl_TexCoord[0].st).a;",
 		"		if (a < 0.99379) {",
-		"			a = 1.0-asin(2.0*(a+0.00621)-1.0)*2.0/3.14159265358979323846264;",
-		"			a = clamp(a*size*(1.0-size*0.016), 0.0, 1.0);",
+		"			a = 1.0-texture1D(asinTexture, a+0.00621).a;",
+		"			a = clamp(a*size*(1.0-size*0.01), 0.0, 1.0) * smoothstep(0.0, softness, a);",
 		"			gl_FragColor = color*a + source*(1.0-color.a*a);",
 		"		} else {",
 		"			gl_FragColor = source;",
@@ -323,20 +393,22 @@ public class Border {
 	public static final String[] INSIDE_BORDER_ONLY = {
 		"uniform sampler2D texBlurred;",
 		"uniform sampler2D texSource;",
+		"uniform sampler1D asinTexture;",
 		"uniform float threshold;",
 		"uniform float size;",
+		"uniform float softness;",
 		"uniform vec4 color;",
 		"",
 		"void main(void)",
 		"{",
 		"	vec4 source = texture2D(texSource, gl_TexCoord[1].st);",
-		"	if (source.a <= threshold) {",
+		"	if (source.a < threshold) {",
 		"		gl_FragColor = color*(source.a/threshold);",
 		"	} else {",
 		"		float a = texture2D(texBlurred, gl_TexCoord[0].st).a;",
 		"		if (a < 0.99379) {",
-		"			a = 1.0-asin(2.0*(a+0.00621)-1.0)*2.0/3.14159265358979323846264;",
-		"			a = clamp(a*size*(1.0-size*0.016), 0.0, 1.0);",
+		"			a = 1.0-texture1D(asinTexture, a+0.00621).a;",
+		"			a = clamp(a*size*(1.0-size*0.01), 0.0, 1.0) * smoothstep(0.0, softness, a);",
 		"			gl_FragColor = color*a;",
 		"		} else {",
 		"			gl_FragColor = vec4(0.0);",
@@ -349,8 +421,10 @@ public class Border {
 	public static final String[] BOTH = {
 		"uniform sampler2D texBlurred;",
 		"uniform sampler2D texSource;",
+		"uniform sampler1D asinTexture;",
 		"uniform float threshold;",
 		"uniform float size;",
+		"uniform float softness;",
 		"uniform vec4 color;",
 		"",
 		"void main(void)",
@@ -359,11 +433,11 @@ public class Border {
 		"	float a = texture2D(texBlurred, gl_TexCoord[0].st).a;",
 		"	if (a > 0.00621 && a < 0.99379) {",
 		"		if (a < 0.5) {",
-		"			a = 1.0+asin(2.0*(a-0.00621)-1.0)*2.0/3.14159265358979323846264;",
+		"			a = 1.0+texture1D(asinTexture, a-0.00621).a;",
 		"		} else {",
-		"			a = 1.0-asin(2.0*(a+0.00621)-1.0)*2.0/3.14159265358979323846264;",
+		"			a = 1.0-texture1D(asinTexture, a+0.00621).a;",
 		"		}",
-		"		a = clamp(a*size*(1.0-size*0.016), 0.0, 1.0);",
+		"		a = clamp(a*size*(1.0-size*0.01), 0.0, 1.0) * smoothstep(0.0, softness, a);",
 		"		gl_FragColor = color*a + source*(1.0-color.a*a);",
 		"	} else {",
 		"		gl_FragColor = source;",
@@ -375,8 +449,10 @@ public class Border {
 	public static final String[] BOTH_BORDER_ONLY = {
 		"uniform sampler2D texBlurred;",
 		"uniform sampler2D texSource;",
+		"uniform sampler1D asinTexture;",
 		"uniform float threshold;",
 		"uniform float size;",
+		"uniform float softness;",
 		"uniform vec4 color;",
 		"",
 		"void main(void)",
@@ -385,11 +461,11 @@ public class Border {
 		"	float a = texture2D(texBlurred, gl_TexCoord[0].st).a;",
 		"	if (a > 0.00621 && a < 0.99379) {",
 		"		if (a < 0.5) {",
-		"			a = 1.0+asin(2.0*(a-0.00621)-1.0)*2.0/3.14159265358979323846264;",
+		"			a = 1.0+texture1D(asinTexture, a-0.00621).a;",
 		"		} else {",
-		"			a = 1.0-asin(2.0*(a+0.00621)-1.0)*2.0/3.14159265358979323846264;",
+		"			a = 1.0-texture1D(asinTexture, a+0.00621).a;",
 		"		}",
-		"		a = clamp(a*size*(1.0-size*0.016), 0.0, 1.0);",
+		"		a = clamp(a*size*(1.0-size*0.01), 0.0, 1.0) * smoothstep(0.0, softness, a);",
 		"		gl_FragColor = color*a;",
 		"	} else {",
 		"		gl_FragColor = vec4(0.0);",
