@@ -217,13 +217,13 @@ public class Glow {
 
 	@ShaderSource
 	public static final String[] OCOLORS_THRESHOLD = {
-		"uniform sampler2DRect texture;",
+		"uniform sampler2D texture;",
 		"uniform float upper;",
 		"uniform float lower;",
 		"",
 		"void main(void)",
 		"{",
-		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
+		"	vec4 color = texture2D(texture, gl_TexCoord[0].st);",
 		"	float a = color.a;",
 		"	vec3 rgb = (a != 0.0) ? color.rgb/a : vec3(0.0);",
 		"",
@@ -235,7 +235,7 @@ public class Glow {
 
 	@ShaderSource
 	public static final String[] ABCOLORS_THRESHOLD = {
-		"uniform sampler2DRect texture;",
+		"uniform sampler2D texture;",
 		"uniform float upper;",
 		"uniform float lower;",
 		"",
@@ -243,7 +243,7 @@ public class Glow {
 		"",
 		"void main(void)",
 		"{",
-		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
+		"	vec4 color = texture2D(texture, gl_TexCoord[0].st);",
 		"",
 		"	float luma = dot(color.rgb, lumaVec);",
 		"	luma = clamp((luma - lower) / (upper - lower), 0.0, 1.0);",
@@ -254,13 +254,13 @@ public class Glow {
 
 	@ShaderSource
 	public static final String[] ALPHA_BASE_THRESHOLD = {
-		"uniform sampler2DRect texture;",
+		"uniform sampler2D texture;",
 		"uniform float upper;",
 		"uniform float lower;",
 		"",
 		"void main(void)",
 		"{",
-		"	float a = texture2DRect(texture, gl_TexCoord[0].st).a;",
+		"	float a = texture2D(texture, gl_TexCoord[0].st).a;",
 		"",
 		"	a = clamp((a - lower) / (upper - lower), 0.0, 1.0);",
 		"",
@@ -304,7 +304,7 @@ public class Glow {
 
 	@ShaderSource
 	public static final String[] SAWTOOTH = {
-		"uniform sampler2DRect texture;",
+		"uniform sampler2D texture;",
 		"uniform float loops;",
 		"uniform float phase;",
 		"uniform float midpoint;",
@@ -314,7 +314,7 @@ public class Glow {
 		"",
 		"void main(void)",
 		"{",
-		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
+		"	vec4 color = texture2D(texture, gl_TexCoord[0].st);",
 		"	float a = color.a;",
 		"	float t = phase + a * loops;",
 		"",
@@ -337,7 +337,7 @@ public class Glow {
 
 	@ShaderSource
 	public static final String[] TRIANGLE = {
-		"uniform sampler2DRect texture;",
+		"uniform sampler2D texture;",
 		"uniform float loops;",
 		"uniform float phase;",
 		"uniform float midpoint;",
@@ -347,7 +347,7 @@ public class Glow {
 		"",
 		"void main(void)",
 		"{",
-		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
+		"	vec4 color = texture2D(texture, gl_TexCoord[0].st);",
 		"	float a = color.a;",
 		"	float t = phase + a * loops;",
 		"",
@@ -375,11 +375,11 @@ public class Glow {
 		Color colorA;
 		Color colorB;
 		if (looping == GlowLooping.SAWTOOTH_AB || looping == GlowLooping.TRIANGLE_ABA) {
-			colorA = context.value(this.colorA).clamp();
-			colorB = context.value(this.colorB).clamp();
+			colorA = context.value(this.colorA);
+			colorB = context.value(this.colorB);
 		} else {
-			colorA = context.value(this.colorB).clamp();
-			colorB = context.value(this.colorA).clamp();
+			colorA = context.value(this.colorB);
+			colorB = context.value(this.colorA);
 		}
 		Color colorM = new Color((colorA.r+colorB.r)/2, (colorA.g+colorB.g)/2, (colorA.b+colorB.b)/2);
 
@@ -404,36 +404,36 @@ public class Glow {
 
 	@ShaderSource
 	public static final String[] OCOLORS_INTENSITY = {
-		"uniform sampler2DRect texture;",
+		"uniform sampler2D texture;",
 		"uniform float intensity;",
 		"",
 		"void main(void)",
 		"{",
-		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
+		"	vec4 color = texture2D(texture, gl_TexCoord[0].st);",
 		"	gl_FragColor = vec4(min(color.rgb*intensity, color.a), color.a);",
 		"}"
 	};
 
 	@ShaderSource
 	public static final String[] OCOLORS_INTENSITY_FLOAT = {
-		"uniform sampler2DRect texture;",
+		"uniform sampler2D texture;",
 		"uniform float intensity;",
 		"",
 		"void main(void)",
 		"{",
-		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
+		"	vec4 color = texture2D(texture, gl_TexCoord[0].st);",
 		"	gl_FragColor = vec4(color.rgb*intensity, color.a);",
 		"}"
 	};
 
 	@ShaderSource
 	public static final String[] ABCOLORS_INTENSITY = {
-		"uniform sampler2DRect texture;",
+		"uniform sampler2D texture;",
 		"uniform float intensity;",
 		"",
 		"void main(void)",
 		"{",
-		"	vec4 color = texture2DRect(texture, gl_TexCoord[0].st);",
+		"	vec4 color = texture2D(texture, gl_TexCoord[0].st);",
 		"	gl_FragColor = vec4(color.rgb*intensity, min(color.a*intensity, 1.0));",
 		"}"
 	};
@@ -471,8 +471,8 @@ public class Glow {
 		boolean normal = name.equals("normal");
 
 		return new String[] {
-								"uniform sampler2DRect texDst;",	// glow
-								"uniform sampler2DRect texSrc;",	// orig
+								"uniform sampler2D texDst;",	// glow
+								"uniform sampler2D texSrc;",	// orig
 								"uniform float opacity;",
 				dissolve ?		"uniform float dissolveSeed;" : "",
 								"",
@@ -481,8 +481,8 @@ public class Glow {
 								"",
 								"void main(void)",
 								"{",
-								"	vec4 glow = texture2DRect(texDst, gl_TexCoord[0].st);",
-								"	vec4 orig = texture2DRect(texSrc, gl_TexCoord[1].st);",
+								"	vec4 glow = texture2D(texDst, gl_TexCoord[0].st);",
+								"	vec4 orig = texture2D(texSrc, gl_TexCoord[1].st);",
 				String.format(	"	glow = blend_%s(glow, orig, opacity%s);", name, dissolve ? ", dissolveSeed" : ""),
 				String.format(	"	gl_FragColor = blend_normal(%s, 1.0);", onTop ? "glow, orig" : "orig, glow"),
 								"}"
